# Copyright (C) 2008-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestSubRip(aeidon.TestCase):

    def setup_method(self, method):
        self.container = aeidon.containers.new("subrip")

    def test_attributes(self):
        assert hasattr(self.container, "x1")
        assert hasattr(self.container, "y1")
        assert hasattr(self.container, "x2")
        assert hasattr(self.container, "y2")


class TestSubStationAlpha(aeidon.TestCase):

    def setup_method(self, method):
        self.container = aeidon.containers.new("ssa")

    def test_attributes(self):
        assert hasattr(self.container, "marked")
        assert hasattr(self.container, "layer")
        assert hasattr(self.container, "style")
        assert hasattr(self.container, "name")
        assert hasattr(self.container, "margin_l")
        assert hasattr(self.container, "margin_r")
        assert hasattr(self.container, "margin_v")
        assert hasattr(self.container, "effect")


class TestModule(aeidon.TestCase):

    def test_new(self):
        aeidon.containers.new("ssa")
        aeidon.containers.new("subrip")

    def test_new__value_error(self):
        self.raises(ValueError, aeidon.containers.new, "xxx")



# Copyright (C) 2006 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Text markup for the TMPlayer format."""

import aeidon

__all__ = ("TMPlayer",)


class TMPlayer(aeidon.Markup):

    """Text markup for the TMPlayer format.

    TMPlayer format is assumed to contain no markup.
    """

    format = aeidon.formats.TMPLAYER



# Copyright (C) 2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Metaclass for design-by-contract_ pre- and postcondition checks.

.. _design-by-contract:
   http://www.eiffel.com/developers/design_by_contract.html
"""

import aeidon
import functools

__all__ = ("Contractual",)


def _ensured(ensure_func):
    """Decorator for checking a postcondition."""
    def outer_wrapper(function):
        @functools.wraps(function)
        def inner_wrapper(*args, **kwargs):
            value = function(*args, **kwargs)
            ensure_func(args[0], value, *args[1:], **kwargs)
            return value
        return inner_wrapper
    return outer_wrapper

def _invariated(invariant_func):
    """Decorator for checking a class invariant."""
    def outer_wrapper(function):
        @functools.wraps(function)
        def inner_wrapper(*args, **kwargs):
            value = function(*args, **kwargs)
            invariant_func(args[0])
            return value
        return inner_wrapper
    return outer_wrapper

def _required(require_func):
    """Decorator for checking a precondition."""
    def outer_wrapper(function):
        @functools.wraps(function)
        def inner_wrapper(*args, **kwargs):
            require_func(*args, **kwargs)
            return function(*args, **kwargs)
        return inner_wrapper
    return outer_wrapper


class Contractual(type):

    """Metaclass for design-by-contract_ pre- and postcondition checks.

    Method calls are automatically wrapped in ``METHOD_NAME_require``
    precondition call, ``METHOD_NAME_ensure`` postcondition call and public
    method calls in ``_invariant`` class invariant check call if such methods
    exist. The require method receives the same arguments as method, the ensure
    method will in addition receive method's return value as its first argument
    after self.

    Preconditions may be weakened; only the first precondition found in the
    inheritance tree is used. Postconditions and class invariants may be
    strengthened; all postconditions and class invariant checks found in the
    inheritance tree will be used.

    This is a debug metaclass that is in use only if :data:`aeidon.debug` is
    ``True``.

    .. _design-by-contract:
       http://www.eiffel.com/developers/design_by_contract.html
    """

    def __new__(meta, class_name, bases, dic):
        """Return instance with method calls wrapped in condition calls."""
        new_dict = dic.copy()
        for name, attr in dic.items():
            if not callable(attr): continue

            require_name = "%s_require" % name
            if name.startswith("__"):
                require_name = "_%s%s" % (class_name, require_name)
            if require_name in dic:
                require_func = dic[require_name]
                attr = _required(require_func)(attr)
            elif not name.startswith("__"):
                # If a precondition is not defined in this class, look for
                # a precodition among ancestors and use the first one found.
                for base in bases:
                    if hasattr(base, require_name):
                        require_func = getattr(base, require_name)
                        attr = _required(require_func)(attr)
                        break

            ensure_name = "%s_ensure" % name
            if name.startswith("__"):
                ensure_name = "_%s%s" % (class_name, ensure_name)
            if ensure_name in dic:
                ensure_func = dic[ensure_name]
                attr = _ensured(ensure_func)(attr)
            if not name.startswith("__"):
                # Check all postconditions defined by ancestors.
                for base in bases:
                    if hasattr(base, ensure_name):
                        ensure_func = getattr(base, ensure_name)
                        attr = _ensured(ensure_func)(attr)

            invariant_name = "_invariant"
            if invariant_name in dic:
                invariant_func = dic[invariant_name]
                attr = _invariated(invariant_func)(attr)
            # Check all class invariants defined by ancestors.
            for base in bases:
                if hasattr(base, invariant_name):
                    invariant_func = getattr(base, invariant_name)
                    attr = _invariated(invariant_func)(attr)

            new_dict[name] = attr

        return type.__new__(meta, class_name, bases, new_dict)

    if not aeidon.debug: del __new__



# Copyright (C) 2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol


class TestModule(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()

    def test_field_actions(self):
        for field in gaupol.fields:
            name = gaupol.field_actions[field]
            assert self.application.get_action(name) is not None

    def test_framerate_actions(self):
        for framerate in aeidon.framerates:
            name = gaupol.framerate_actions[framerate]
            assert self.application.get_action(name) is not None

    def test_mode_actions(self):
        for mode in aeidon.modes:
            name = gaupol.mode_actions[mode]
            assert self.application.get_action(name) is not None



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Text markup for the Sub Station Alpha format."""

import aeidon
import re

__all__ = ("SubStationAlpha",)


class SubStationAlpha(aeidon.Markup):

    """Text markup for the Sub Station Alpha format.

    Sub Station Alpha format contains a lot of markup tags of which the
    following are of interest to us. The generic reset ``{\\r}`` is used to
    revert to regular text, i.e. to close all open tags. Most of the tagging
    methods, e.g. :meth:`colorize`, leave tags unclosed, instead of explicitly
    closing them and possibly other tags with ``{\\r}``. Usually this is not a
    problem as such tags tend to be applied to the whole subtitle.

     * ``{\\b1}...........{\\b0}``
     * ``{\\i1}...........{\\i0}``
     * ``{\\fnNAME}............``
     * ``{\\fsPOINTS}..........``
     * ``{\\c&HBBGGRR&}........``
     * ``.................{\\r}``

     The hexadecimal color value is in reverse order, ``BBGGRR`` instead of the
     normal ``RRGGBB``. Furthermore, leading zeros can be omitted,
     e.g. ``ff00`` can be used instead of ``00ff00``.
    """

    _closing_pattern = r"\{\\([bi])0\}"
    _flags = re.DOTALL | re.MULTILINE | re.UNICODE | re.IGNORECASE
    _opening_pattern = r"\{\\(?![bi]0)(b|i|c|fn|fs).*?\}"
    _reset_pattern = r"\{\\r\}"
    format = aeidon.formats.SSA

    def _main_decode(self, text):
        """Return `text` with decodable markup decoded."""
        text = self._decode_b(text, r"\{\\b1\}(.*?)\{\\b[0\\]\}", 1)
        text = self._decode_c(text, r"\{\\c#(.+?)\}(.*?)\{\\c\\\}", 1, 2)
        text = self._decode_f(text, r"\{\\fn(.+?)\}(.*?)\{\\fn\\\}", 1, 2)
        text = self._decode_i(text, r"\{\\i1\}(.*?)\{\\i[0\\]\}", 1)
        return self._decode_s(text, r"\{\\fs(\d+)\}(.*?)\{\\fs\\\}", 1, 2)

    def _post_decode_ensure(self, value, text):
        regex = self._get_regex(r"\{\\.*?\}")
        assert regex.search(value) is None

    def _post_decode(self, text):
        """Return `text` with markup finalized after decoding."""
        # Remove all unsupported markup tags.
        return self._substitute(text, r"\{\\.*?\}", "")

    def _pre_decode(self, text):
        """Return `text` with markup prepared for decoding."""
        text = self._pre_decode_break(text)
        text = self._pre_decode_reset(text)
        return self._pre_decode_color(text)

    def _pre_decode_break(self, text):
        """Return `text` with combined markup tags separated.

        For example, ``{\\b1\\i1}`` is replaced with ``{\\b1}{\\i1}``.
        """
        parts = text.split("\\")
        for i in range(1, len(parts)):
            text_so_far = "\\".join(parts[:i])
            if text_so_far.endswith("{"): continue
            opening_index = text_so_far.rfind("{")
            closing_index = text_so_far.rfind("}")
            if opening_index > closing_index:
                parts[i - 1] += "}{"
        return "\\".join(parts)

    def _pre_decode_color_ensure(self, value, text):
        regex = self._get_regex(r"\{\\c&H([0-9a-fA-F]*)&\}")
        assert regex.search(value) is None

    def _pre_decode_color(self, text):
        """Return `text` with colors converted to standard hexadecimal form.

        Color tags are converted from ``{\\c&HBBGGRR&}`` to ``{\\c#RRGGBB}``.
        """
        pattern = r"\{\\c&H([0-9a-fA-F]*)&\}"
        regex = self._get_regex(pattern)
        match = regex.search(text)
        if match is None: return text
        color = ("%06s" % match.group(1)).replace(" ", "0")
        color = "%s%s%s" % (color[4:], color[2:4], color[:2])
        text = regex.sub(r"{\\c#%s}" % color, text, 1)
        return self._pre_decode_color(text)

    def _pre_decode_reset_ensure(self, value, text):
        regex = self._get_regex(self._reset_pattern)
        assert regex.search(value) is None

    def _pre_decode_reset(self, text):
        """Return `text` with all markup tags closed explicitly.

        Tags of form ``{\\nameVALUE}`` are closed with ``{\\name\\}``.
        The returned text will not contain reset ``{\\r}`` tags.
        """
        re_opening = self._get_regex(self._opening_pattern)
        re_closing = self._get_regex(self._closing_pattern)
        re_reset = self._get_regex(self._reset_pattern)
        parts = re_reset.split(text + "{\\r}")
        for i, part in enumerate(parts):
            opening_matches = [x for x in re_opening.finditer(part)]
            closing_matches = [x for x in re_closing.finditer(part)]
            # Find out which tags have already been closed.
            for j in reversed(range(len(closing_matches))):
                closing_core = closing_matches[j].group(1)
                for k in range(len(opening_matches)):
                    opening_core = opening_matches[k].group(1)
                    if opening_core == closing_core:
                        opening_matches.pop(k)
                        break
            # Add artificial closing tags to close remaining tags.
            for j in reversed(range(len(opening_matches))):
                parts[i] += "{\\%s\\}" % opening_matches[j].group(1)
        return "".join(parts)

    def bolden(self, text, bounds=None):
        """Return bolded `text`."""
        a, z = bounds or (0, len(text))
        return "".join((text[:a], "{\\b1}%s{\\b0}" % text[a:z], text[z:]))

    def colorize(self, text, color, bounds=None):
        """Return `text` colorized to hexadecimal value."""
        a, z = bounds or (0, len(text))
        # Reverse the color value from RRGGBB to BBGGRR.
        color = "%s%s%s" % (color[4:], color[2:4], color[:2])
        target = "{\\c&H%s&}%s" % (color, text[a:z])
        return "".join((text[:a], target, text[z:]))

    def fontify(self, text, font, bounds=None):
        """Return `text` changed to `font`."""
        a, z = bounds or (0, len(text))
        target = "{\\fn%s}%s" % (font, text[a:z])
        return "".join((text[:a], target, text[z:]))

    @property
    def italic_tag(self):
        """Regular expression for an italic markup tag."""
        return self._get_regex(r"\{\\i[01]\}")

    def italicize(self, text, bounds=None):
        """Return italicized `text`."""
        a, z = bounds or (0, len(text))
        return "".join((text[:a], "{\\i1}%s{\\i0}" % text[a:z], text[z:]))

    def scale(self, text, size, bounds=None):
        """Return `text` scaled to `size`."""
        a, z = bounds or (0, len(text))
        target = "{\\fs%s}%s" % (str(size), text[a:z])
        return "".join((text[:a], target, text[z:]))

    @property
    def tag(self):
        """Regular expression for any markup tag."""
        return self._get_regex(r"\{\\.*?\}")



# Copyright (C) 2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol
import gtk


class TestExtension(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        class PuppetExtension(gaupol.Extension): pass
        self.extension = PuppetExtension()

    def test_setup_method(self):
        self.extension.setup(self.application)

    def test_show_help(self):
        self.raises(NotImplementedError,
                    self.extension.show_help)

    def test_show_preferences_dialog(self):
        self.raises(NotImplementedError,
                    self.extension.show_preferences_dialog,
                    gtk.Window())

    def test_teardown_method(self):
        self.extension.teardown(self.application)

    def test_update(self):
        self.extension.update(self.application,
                              self.application.get_current_page())



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for selecting properties to save multiple files with."""

import aeidon
import gaupol
import gtk
import os
_ = aeidon.i18n._

__all__ = ("MultiSaveDialog",)


class MultiSaveDialog(gaupol.FileDialog):

    """Dialog for selecting properties to save multiple files with."""

    _widgets = ("encoding_combo",
                "filechooser_button",
                "format_combo",
                "newline_combo")

    def __init__(self, parent, application):
        """Initialize a :class:`MultiSaveDialog` object."""
        gaupol.FileDialog.__init__(self, "multisave-dialog.ui")
        self.application = application
        self._init_format_combo()
        self._init_encoding_combo()
        self._init_newline_combo()
        width = gaupol.util.char_to_px(60)
        self._filechooser_button.set_size_request(width, -1)
        self._init_values()
        self.set_transient_for(parent)

    def _init_format_combo(self):
        """Initialize the format combo box."""
        store = gtk.ListStore(str)
        self._format_combo.set_model(store)
        for name in (x.label for x in aeidon.formats):
            store.append((name,))
        view = self._format_combo.get_child()
        view.set_displayed_row(0)
        renderer = gtk.CellRendererText()
        self._format_combo.pack_start(renderer, True)
        self._format_combo.add_attribute(renderer, "text", 0)

    def _init_newline_combo(self):
        """Initialize the newline combo box."""
        store = gtk.ListStore(str)
        self._newline_combo.set_model(store)
        for name in (x.label for x in aeidon.newlines):
            store.append((name,))
        view = self._newline_combo.get_child()
        view.set_displayed_row(0)
        renderer = gtk.CellRendererText()
        self._newline_combo.pack_start(renderer, True)
        self._newline_combo.add_attribute(renderer, "text", 0)

    def _init_values(self):
        """Initialize default values for widgets."""
        pages = [x for x in self.application.pages
                 if x.project.main_file is not None]

        sfile = pages[0].project.main_file
        self.set_directory(os.path.dirname(sfile.path))
        self.set_format(sfile.format)
        self.set_encoding(sfile.encoding)
        self.set_newline(sfile.newline)

    def _on_response(self, dialog, response):
        """Save default values for widgets."""
        gaupol.conf.file.encoding = self.get_encoding()
        gaupol.conf.file.format = self.get_format()
        gaupol.conf.file.newline = self.get_newline()
        if response != gtk.RESPONSE_OK: return
        gaupol.util.set_cursor_busy(self._dialog)
        self._save_all_documents_as()
        gaupol.util.set_cursor_normal(self._dialog)

    @aeidon.deco.silent(gaupol.Default)
    def _save_all_documents_as(self):
        """Save all documents with selected properties."""
        pages = [x for x in self.application.pages
                 if x.project.main_file is not None]

        files = [None for x in pages]
        for i, page in enumerate(pages):
            path = os.path.basename(page.project.main_file.path)
            path = aeidon.util.replace_extension(path, self.get_format())
            path = os.path.join(self.get_directory(), path)
            files[i] = aeidon.files.new(self.get_format(),
                                        path,
                                        self.get_encoding(),
                                        self.get_newline())

        overwrite_count = sum(os.path.isfile(x.path) for x in files)
        if overwrite_count > 0:
            self._show_overwrite_question_dialog(overwrite_count,
                                                 self.get_directory())

        for i, page in enumerate(pages):
            self.application.save_main_as(page, files[i])

    def _show_overwrite_question_dialog(self, overwrite_count, path):
        """Show a question dialog if about to overwrite files.

        Raise :exc:`gaupol.Default` if opening cancelled.
        """
        title = _("%d of the files to be saved already exist. "
                  "Do you want to replace them?") % overwrite_count

        message = _('The files already exist in "%s". '
                    'Replacing them will overwrite their contents.') % path

        dialog = gaupol.QuestionDialog(self._dialog, title, message)
        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_NO)
        dialog.add_button(_("_Replace"), gtk.RESPONSE_YES)
        dialog.set_default_response(gtk.RESPONSE_YES)
        response = gaupol.util.flash_dialog(dialog)
        gaupol.util.raise_default(response != gtk.RESPONSE_YES)

    def get_format(self):
        """Return the selected format."""
        index = self._format_combo.get_active()
        return aeidon.formats[index]

    def get_directory(self):
        """Return the selected directory."""
        return self._filechooser_button.get_filename()

    def get_newline(self):
        """Return the selected newline."""
        index = self._newline_combo.get_active()
        return aeidon.newlines[index]

    def set_directory(self, path):
        """Set the selected directory."""
        return self._filechooser_button.set_filename(path)

    def set_format(self, format):
        """Set the selected format."""
        if format is None: return
        self._format_combo.set_active(format)

    def set_newline(self, newline):
        """Set the selected newline."""
        if newline is None: return
        self._newline_combo.set_active(newline)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Subtitle text markup converter."""

import aeidon

__all__ = ("MarkupConverter",)


class MarkupConverter(object):

    """Subtitle text markup converter."""

    def __init__(self, from_format, to_format):
        """Initialize a :class:`MarkupConverter` instance.

        `from_format` and `to_format` should be :attr:`aeidon.formats`
        enumeration items.
        """
        self._from = aeidon.tags.new(from_format)
        self._to = aeidon.tags.new(to_format)

    def convert(self, text):
        """Return `text` with markup converted."""
        return self._to.encode(self._from.decode(text))



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestRevertableAction(aeidon.TestCase):

    def setup_method(self, method):
        self.revert_action_call_count = 0
        self.revert_action_register = None
        def revert_action(x, y, z=None, register=-1):
            self.revert_action_call_count += 1
            self.revert_action_register = register
            assert (x, y, z) == (0, 1, 2)
        self.action = aeidon.RevertableAction(description="")
        self.action.register = aeidon.registers.DO
        self.action.docs = (aeidon.documents.MAIN,)
        self.action.revert_args = (0, 1)
        self.action.revert_kwargs = {"z": 2}
        self.action.revert_function = revert_action

    def test_revert__do(self):
        self.action.register = aeidon.registers.DO
        self.action.revert()
        assert self.revert_action_call_count == 1
        register = self.revert_action_register
        assert register == aeidon.registers.UNDO

    def test_revert__redo(self):
        self.action.register = aeidon.registers.REDO
        self.action.revert()
        assert self.revert_action_call_count == 1
        register = self.revert_action_register
        assert register == aeidon.registers.UNDO

    def test_revert__undo(self):
        self.action.register = aeidon.registers.UNDO
        self.action.revert()
        assert self.revert_action_call_count == 1
        register = self.revert_action_register
        assert register == aeidon.registers.REDO


class TestRevertableActionGroup(aeidon.TestCase):

    def setup_method(self, method):
        self.action_group = aeidon.RevertableActionGroup(actions=())
        self.action_group.description = ""



# Copyright (C) 2011 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for generating subtitles by voice and speech recognition."""

from __future__ import division

import aeidon
import gaupol
import gtk
import itertools
import os

__all__ = ("SpeechRecognitionDialog",)


class SpeechRecognitionDialog(gaupol.BuilderDialog):

    """Dialog for generating subtitles by voice and speech recognition."""

    __metaclass__ = aeidon.Contractual

    _widgets = ("acoustic_button",
                "advance_spin",
                "default_model_check",
                "dialog",
                "dict_button",
                "lang_button",
                "model_table",
                "noise_spin",
                "options_vbox",
                "progressbar",
                "revert_button",
                "silence_spin",
                "video_button",
                )

    def __init___require(self, parent, application):
        assert gaupol.util.gst_available()
        assert gaupol.util.pocketsphinx_available()

    def __init__(self, parent, application):
        """Initialize a :class:`SpeechRecognitionDialog` object."""
        gaupol.BuilderDialog.__init__(self, "recognition-dialog.ui")
        self.application = application
        self._page = None
        self._pipeline = None
        self._starts = []
        self._stops = []
        self._text = None
        self._texts = []
        self._init_values()
        self._init_sensitivities()
        width = gaupol.util.char_to_px(42)
        self._acoustic_button.set_size_request(width, -1)
        self._update_response_sensitivities()
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_CLOSE)

    def _clear_attributes(self):
        """Clear values of result attributes."""
        self._pipeline = None
        self._starts = []
        self._stops = []
        self._text = None
        self._texts = []

    def _get_filesrc_definition_require(self):
        assert self._video_button.get_filename() is not None

    def _get_filesrc_definition(self):
        """Return ``filesrc`` definition for :func:`gst.parse_launch`."""
        path = self._video_button.get_filename()
        return 'filesrc location="%s" ' % path

    def _get_pipeline_definition(self):
        """Return pipeline definition for :func:`gst.parse_launch`."""
        return (self._get_filesrc_definition()
                + "! decodebin2 "
                + "! audioconvert "
                + "! audioresample "
                + self._get_vader_definition()
                + self._get_pocketsphinx_definition()
                + "! fakesink"
                )

    def _get_pocketsphinx_definition_require(self):
        if not self._default_model_check.get_active():
            assert self._acoustic_button.get_current_folder() is not None
            assert self._dict_button.get_filename() is not None
            assert self._lang_button.get_filename() is not None

    def _get_pocketsphinx_definition(self):
        """Return ``pocketsphinx`` definition for :func:`gst.parse_launch`."""
        definition = "! pocketsphinx name=pocketsphinx "
        if self._default_model_check.get_active():
            return definition
        return definition + 'hmm="%s" dict="%s" lm="%s" ' % (
            self._acoustic_button.get_current_folder(),
            self._dict_button.get_filename(),
            self._lang_button.get_filename())

    def _get_vader_definition(self):
        """Return ``vader`` definition for :func:`gst.parse_launch`."""
        # Convert noise level from spin button range [0,32768] to gstreamer
        # element's range [0,1]. Likewise, convert silence from spin button's
        # milliseconds to gstreamer element's nanoseconds.
        noise = self._noise_spin.get_value_as_int() / 32768
        silence = self._silence_spin.get_value_as_int() * 1000000
        return ("! vader "
                + "name=vader "
                + "auto-threshold=false "
                + "threshold=%.9f " % noise
                + "run-length=%d " % silence
                )

    def _init_sensitivities(self):
        """Initialize widget sensitivities."""
        self.set_response_sensitive(gtk.RESPONSE_HELP, True)
        self.set_response_sensitive(gtk.RESPONSE_CANCEL, False)
        self.set_response_sensitive(gtk.RESPONSE_OK, False)
        self.set_response_sensitive(gtk.RESPONSE_CLOSE, True)

    def _init_values(self):
        """Initialize default values for widgets."""
        conf = gaupol.conf.speech_recognition
        self._noise_spin.set_value(conf.noise_level)
        self._silence_spin.set_value(conf.silence_length)
        self._advance_spin.set_value(conf.advance_length)
        self._default_model_check.set_active(not conf.use_custom_models)
        self._model_table.set_sensitive(conf.use_custom_models)
        if os.path.isdir(conf.acoustic_model):
            self._acoustic_button.set_current_folder(conf.acoustic_model)
        if os.path.isfile(conf.phonetic_dict):
            self._dict_button.set_filename(conf.phonetic_dict)
        if os.path.isfile(conf.lang_model):
            self._lang_button.set_filename(conf.lang_model)

    def _append_subtitle_require(self, index):
        if index < 0:
            index += len(self._starts)
        assert index in range(len(self._starts))

    def _append_subtitle(self, index):
        """Create subtitle from `index` and append to page."""
        if index < 0:
            index += len(self._starts)
        advance = gaupol.conf.speech_recognition.advance_length
        advance = float(advance / 1000) # ms to s
        subtitle = aeidon.Subtitle(mode=aeidon.modes.TIME)
        start = max((self._stops[index - 1] if index > 0 else 0.0),
                    (self._starts[index] - advance))

        subtitle.start = float(start)
        subtitle.end = float(self._stops[index])
        subtitle.main_text = self._texts[index] or ("[%d]" % (index + 1))
        indices = (len(self._page.project.subtitles),)
        self._page.project.insert_subtitles(indices,
                                            (subtitle,),
                                            register=None)

    def _on_acoustic_button_current_folder_changed(self, file_button):
        """Save the acoustic model directory setting."""
        value = file_button.get_current_folder()
        if value is None: return
        gaupol.conf.speech_recognition.acoustic_model = value
        self._update_response_sensitivities()

    def _on_advance_spin_value_changed(self, spin_button):
        """Save advance length setting."""
        value = spin_button.get_value_as_int()
        gaupol.conf.speech_recognition.advance_length = value

    def _on_bus_message_application(self, bus, message):
        """Process application messages from the bus."""
        import gst
        name = message.structure.get_name()
        if name == "start":
            start = message.structure["start"]
            self._starts.append(start)
            if self._text is not None:
                # Store previous text.
                self._texts[-1] = self._text
                self._text = None
            self._stops.append(None)
            self._texts.append(None)
            duration = self._pipeline.query_duration(gst.FORMAT_TIME, None)[0]
            duration = duration / 1000000000 # ns to s
            fraction = start / duration
            self._progressbar.set_fraction(fraction)
            if len(self._starts) > 1:
                # Append previous subtitle to page.
                self._append_subtitle(-2)
        if name == "stop":
            stop = message.structure["stop"]
            self._stops[-1] = stop
        if name == "text":
            text = message.structure["text"]
            if not isinstance(text, unicode):
                text = unicode(text, errors="replace")
            self._text = text

    def _on_bus_message_eos(self, bus, message):
        """Flush remaining subtitles to page."""
        if self._text is not None:
            # Store previous text.
            self._texts[-1] = self._text
            self._text = None
        self._progressbar.set_fraction(1)
        if self._starts and self._stops[-1] is not None:
            self._append_subtitle(-1)
        self._stop_speech_recognition()

    def _on_default_model_check_toggled(self, toggle_button):
        """Save default model usage setting."""
        value = toggle_button.get_active()
        gaupol.conf.speech_recognition.use_custom_models = not value
        self._model_table.set_sensitive(not value)
        self._update_response_sensitivities()

    def _on_dict_button_file_set(self, file_button):
        """Save the phonetic dictionary file setting."""
        value = file_button.get_filename()
        if value is None: return
        gaupol.conf.speech_recognition.phonetic_dict = value
        self._update_response_sensitivities()

    def _on_lang_button_file_set(self, file_button):
        """Save the language model file setting."""
        value = file_button.get_filename()
        if value is None: return
        gaupol.conf.speech_recognition.lang_model = value
        self._update_response_sensitivities()

    def _on_noise_spin_value_changed(self, spin_button):
        """Save noise level setting."""
        value = spin_button.get_value_as_int()
        gaupol.conf.speech_recognition.noise_level = value

    def _on_pocketsphinx_result(self, sphinx, text, uttid):
        """Send recognized text as a message on the bus."""
        import gst
        struct = gst.Structure("text")
        struct.set_value("text", text)
        struct.set_value("uttid", uttid)
        sphinx.post_message(gst.message_new_application(sphinx, struct))

    def _on_response(self, dialog, response):
        """Handle responses without destroying dialog."""
        if response == gtk.RESPONSE_HELP:
            gaupol.util.show_uri(gaupol.SPEECH_RECOGNITION_HELP_URL)
            self.stop_emission("response")
        if response == gtk.RESPONSE_CANCEL:
            self._stop_speech_recognition()
            self.stop_emission("response")
        if response == gtk.RESPONSE_OK:
            self._recognize_speech()
            self.stop_emission("response")

    def _on_revert_button_clicked(self, *args):
        """Revert voice recognition parameters to their defaults."""
        query = lambda x: gaupol.conf.query_default("speech_recognition", x)
        self._noise_spin.set_value(query("noise_level"))
        self._silence_spin.set_value(query("silence_length"))
        self._advance_spin.set_value(query("advance_length"))

    def _on_silence_spin_value_changed(self, spin_button):
        """Save silence length setting."""
        value = spin_button.get_value_as_int()
        gaupol.conf.speech_recognition.silence_length = value

    def _on_vader_start(self, vader, pos):
        """Send start position as a message on the bus."""
        import gst
        struct = gst.Structure("start")
        pos = pos / 1000000000 # ns to s
        struct.set_value("start", pos)
        vader.post_message(gst.message_new_application(vader, struct))

    def _on_vader_stop(self, vader, pos):
        """Send stop position as a message on the bus."""
        import gst
        struct = gst.Structure("stop")
        pos = pos / 1000000000 # ns to s
        struct.set_value("stop", pos)
        vader.post_message(gst.message_new_application(vader, struct))

    def _on_video_button_file_set(self, file_button):
        """Update response sensitivities."""
        self._update_response_sensitivities()

    def _prepare_page(self):
        """Prepare page properties for speech recognition."""
        if self._page is None:
            self._page = gaupol.Page(self.application.counter.next())
            self.application.add_page(self._page)
        video_path = self._video_button.get_filename()
        self._page.project.video_path = video_path
        # Set project's main_file attribute to a SubRip file so that previewing
        # is possible wihtout a need to save the file. Set the path to
        # VIDEO.#.srt, so that the file can be easily saved to the same
        # directory, but without risk of overwriting anything.
        base = video_path[:video_path.rfind(".")]
        for i in itertools.count(1):
            path = "%s.%d.srt" % (base, i)
            if not os.path.isfile(path): break
        self._page.project.main_file = aeidon.files.new(aeidon.formats.SUBRIP,
                                                        path,
                                                        "utf_8")

        self._page.project.main_changed = 1
        if self._page.project.subtitles:
            indices = range(len(self._page.project.subtitles))
            self._page.project.remove_subtitles(indices, register=None)

    def _recognize_speech(self):
        """Generate subtitles from video or audio file."""
        import gst
        self._set_sensitivities_start()
        self._progressbar.set_fraction(0)
        self._clear_attributes()
        self._pipeline = gst.parse_launch(self._get_pipeline_definition())
        vader = self._pipeline.get_by_name("vader")
        vader.connect("vader-start", self._on_vader_start)
        vader.connect("vader-stop", self._on_vader_stop)
        sphinx = self._pipeline.get_by_name("pocketsphinx")
        sphinx.connect("result", self._on_pocketsphinx_result)
        sphinx.set_property("configured", True)
        bus = self._pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect("message::application", self._on_bus_message_application)
        bus.connect("message::eos", self._on_bus_message_eos)
        self._prepare_page()
        self._pipeline.set_state(gst.STATE_PLAYING)

    def _set_sensitivities_start(self):
        """Set widget sensitivies for speech recognition start."""
        self._options_vbox.set_sensitive(False)
        self.set_response_sensitive(gtk.RESPONSE_HELP, False)
        self.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
        self.set_response_sensitive(gtk.RESPONSE_OK, False)
        self.set_response_sensitive(gtk.RESPONSE_CLOSE, False)

    def _set_sensitivities_stop(self):
        """Set widget sensitivies for speech recognition start."""
        self._options_vbox.set_sensitive(True)
        self.set_response_sensitive(gtk.RESPONSE_HELP, True)
        self.set_response_sensitive(gtk.RESPONSE_CANCEL, False)
        self.set_response_sensitive(gtk.RESPONSE_OK, True)
        self.set_response_sensitive(gtk.RESPONSE_CLOSE, True)

    def _stop_speech_recognition(self):
        """Stop generating subtitles from video or audio file."""
        import gst
        self._pipeline.set_state(gst.STATE_NULL)
        self._set_sensitivities_stop()

    def _update_response_sensitivities(self):
        """Update dialog response sensitivities."""
        sensitive = self._video_button.get_filename() is not None
        if not self._default_model_check.get_active():
            sensitive = (sensitive and
                         self._acoustic_button.get_current_folder() is not None
                         and self._dict_button.get_filename() is not None
                         and self._lang_button.get_filename() is not None)

        self.set_response_sensitive(gtk.RESPONSE_OK, sensitive)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol
import gtk


class TestDurationAdjustDialog(gaupol.TestCase):

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        self.application = self.new_application()
        page = self.application.get_current_page()
        page.view.select_rows((1, 2, 3))
        self.dialog = gaupol.DurationAdjustDialog(self.application.window,
                                                  self.application)

        self.dialog.show()

    def test___init____no_selection(self):
        page = self.application.get_current_page()
        page.view.select_rows(())
        gaupol.conf.duration_adjust.target = gaupol.targets.SELECTED
        self.dialog = gaupol.DurationAdjustDialog(self.application.window,
                                                  self.application)

    def test__on_gap_check_toggled(self):
        self.dialog._gap_check.set_active(True)
        self.dialog._gap_check.set_active(False)
        self.dialog._gap_check.set_active(True)

    def test__on_lengthen_check_toggled(self):
        self.dialog._lengthen_check.set_active(True)
        self.dialog._lengthen_check.set_active(False)
        self.dialog._lengthen_check.set_active(True)

    def test__on_max_check_toggled(self):
        self.dialog._max_check.set_active(True)
        self.dialog._max_check.set_active(False)
        self.dialog._max_check.set_active(True)

    def test__on_min_check_toggled(self):
        self.dialog._min_check.set_active(True)
        self.dialog._min_check.set_active(False)
        self.dialog._min_check.set_active(True)

    def test__on_response__all(self):
        self.dialog._all_radio.set_active(True)
        self.dialog.response(gtk.RESPONSE_OK)

    def test__on_response__current(self):
        self.dialog._current_radio.set_active(True)
        self.dialog.response(gtk.RESPONSE_OK)

    def test__on_response__selected(self):
        self.dialog._selected_radio.set_active(True)
        self.dialog.response(gtk.RESPONSE_OK)

    def test__on_shorten_check_toggled(self):
        self.dialog._shorten_check.set_active(True)
        self.dialog._shorten_check.set_active(False)
        self.dialog._shorten_check.set_active(True)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Simple subtitle data editing actions for :class:`gaupol.Application`."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._


class ClearTextsAction(gaupol.Action):

    """Clear the selected texts."""

    def __init__(self):
        """Initialize a :class:`ClearTextsAction` object."""
        gaupol.Action.__init__(self, "clear_texts")
        self.props.label = _("Cl_ear")
        self.props.stock_id = gtk.STOCK_CLEAR
        self.props.tooltip = _("Clear the selected texts")
        self.accelerator = "C"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.view.get_selected_rows())
        col = page.view.get_focus()[1]
        aeidon.util.affirm(col is not None)
        aeidon.util.affirm(page.view.is_text_column(col))


class CopyTextsAction(gaupol.Action):

    """Copy the selected texts to the clipboard."""

    def __init__(self):
        """Initialize a :class:`CopyTextsAction` object."""
        gaupol.Action.__init__(self, "copy_texts")
        self.props.label = _("_Copy")
        self.props.stock_id = gtk.STOCK_COPY
        self.props.tooltip = _("Copy the selected texts to the clipboard")
        self.accelerator = "<Control>C"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.view.get_selected_rows())
        col = page.view.get_focus()[1]
        aeidon.util.affirm(col is not None)
        aeidon.util.affirm(page.view.is_text_column(col))


class CutTextsAction(gaupol.Action):

    """Cut the selected texts to the clipboard."""

    def __init__(self):
        """Initialize a :class:`CutTextsAction` object."""
        gaupol.Action.__init__(self, "cut_texts")
        self.props.label = _("Cu_t")
        self.props.stock_id = gtk.STOCK_CUT
        self.props.tooltip = _("Cut the selected texts to the clipboard")
        self.accelerator = "<Control>X"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.view.get_selected_rows())
        col = page.view.get_focus()[1]
        aeidon.util.affirm(col is not None)
        aeidon.util.affirm(page.view.is_text_column(col))


class EditPreferencesAction(gaupol.Action):

    """Configure Gaupol."""

    def __init__(self):
        """Initialize an :class:`EditPreferencesAction` object."""
        gaupol.Action.__init__(self, "edit_preferences")
        self.props.label = _("_Preferences")
        self.props.stock_id = gtk.STOCK_PREFERENCES
        self.props.tooltip = _("Configure Gaupol")
        self.action_group = "main-safe"


class EditNextValueAction(gaupol.Action):

    """Edit the focused column of the next subtitle."""

    def __init__(self):
        """Initialize an :class:`EditNextValueAction` object."""
        gaupol.Action.__init__(self, "edit_next_value")
        self.props.label = _("Edit _Next Cell")
        self.props.short_label = _("Next")
        self.props.stock_id = gtk.STOCK_EDIT
        self.props.tooltip = _("Edit the focused column of the next subtitle")
        self.accelerator = "space"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        row, col = page.view.get_focus()
        aeidon.util.affirm(not None in (row, col))
        aeidon.util.affirm(row < len(page.project.subtitles) - 1)
        column = page.view.get_column(col)
        renderer = column.get_cell_renderers()[0]
        mode = renderer.props.mode
        aeidon.util.affirm(mode == gtk.CELL_RENDERER_MODE_EDITABLE)


class EditValueAction(gaupol.Action):

    """Edit the focused cell."""

    def __init__(self):
        """Initialize an :class:`EditValueAction` object."""
        gaupol.Action.__init__(self, "edit_value")
        self.props.is_important = True
        self.props.label = _("_Edit Cell")
        self.props.short_label = _("Edit")
        self.props.stock_id = gtk.STOCK_EDIT
        self.props.tooltip = _("Edit the focused cell")
        self.accelerator = "Return"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        row, col = page.view.get_focus()
        aeidon.util.affirm(not None in (row, col))
        column = page.view.get_column(col)
        renderer = column.get_cell_renderers()[0]
        mode = renderer.props.mode
        aeidon.util.affirm(mode == gtk.CELL_RENDERER_MODE_EDITABLE)


class ExtendSelectionToBeginningAction(gaupol.Action):

    """Extend the selection up to the first subtitle."""

    def __init__(self):
        """Initialize an :class:`ExtendSelectionToBeginningAction` object."""
        gaupol.Action.__init__(self, "extend_selection_to_beginning")
        self.props.label = _("Extend To _Beginning")
        self.props.tooltip = _("Extend the current selection "
                               "up to the first subtitle")

        self.accelerator = "<Shift><Control>Home"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.subtitles)
        aeidon.util.affirm(page.view.get_selected_rows())


class ExtendSelectionToEndAction(gaupol.Action):

    """Extend the selection up to the last subtitle."""

    def __init__(self):
        """Initialize an :class:`ExtendSelectionToEndAction` object."""
        gaupol.Action.__init__(self, "extend_selection_to_end")
        self.props.label = _("Extend To _End")
        self.props.tooltip = _("Extend the current selection "
                               "up to the last subtitle")

        self.accelerator = "<Shift><Control>End"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.subtitles)
        aeidon.util.affirm(page.view.get_selected_rows())


class InsertSubtitlesAction(gaupol.Action):

    """Insert subtitles."""

    def __init__(self):
        """Initialize an :class:`InsertSubtitlesAction` object."""
        gaupol.Action.__init__(self, "insert_subtitles")
        self.props.label = _("_Insert Subtitles\342\200\246")
        self.props.short_label = _("Insert")
        self.props.stock_id = gtk.STOCK_ADD
        self.props.tooltip = _("Insert new subtitles")
        self.accelerator = "Insert"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        if page.project.subtitles:
            aeidon.util.affirm(page.view.get_selected_rows())


class InvertSelectionAction(gaupol.Action):

    """Invert the current selection."""

    def __init__(self):
        """Initialize an :class:`InvertSelectionAction` object."""
        gaupol.Action.__init__(self, "invert_selection")
        self.props.label = _("_Invert Selection")
        self.props.tooltip = _("Invert the current selection")
        self.accelerator = "<Control>I"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.subtitles)


class MergeSubtitlesAction(gaupol.Action):

    """Merge the selected subtitles."""

    def __init__(self):
        """Initialize a :class:`MergeSubtitlesAction` object."""
        gaupol.Action.__init__(self, "merge_subtitles")
        self.props.label = _("_Merge Subtitles")
        self.props.tooltip = _("Merge the selected subtitles")
        self.accelerator = "M"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        rows = page.view.get_selected_rows()
        aeidon.util.affirm(len(rows) > 1)
        aeidon.util.affirm(list(rows) == range(rows[0], rows[-1] + 1))


class PasteTextsAction(gaupol.Action):

    """Paste texts from the clipboard."""

    def __init__(self):
        """Initialize a :class:`PasteTextsAction` object."""
        gaupol.Action.__init__(self, "paste_texts")
        self.props.label = _("_Paste")
        self.props.stock_id = gtk.STOCK_PASTE
        self.props.tooltip = _("Paste texts from the clipboard")
        self.accelerator = "<Control>V"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(not application.clipboard.is_empty())
        aeidon.util.affirm(page.view.get_selected_rows())
        col = page.view.get_focus()[1]
        aeidon.util.affirm(col is not None)
        aeidon.util.affirm(page.view.is_text_column(col))


class RedoActionAction(gaupol.Action):

    """Redo the last undone action."""

    __gtype_name__ = "RedoActionAction"

    def __init__(self):
        """Initialize a :class:`RedoActionAction` object."""
        gaupol.Action.__init__(self, "redo_action")
        self.props.label = _("_Redo")
        self.props.stock_id = gtk.STOCK_REDO
        self.props.tooltip = _("Redo the last undone action")
        self.accelerator = "<Shift><Control>Z"
        self.action_group = "main-unsafe"
        self.set_tool_item_type(gtk.MenuToolButton)

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.can_redo())


class RemoveSubtitlesAction(gaupol.Action):

    """Remove the selected subtitles."""

    def __init__(self):
        """Initialize a :class:`RemoveSubtitlesAction` object."""
        gaupol.Action.__init__(self, "remove_subtitles")
        self.props.label = _("Rem_ove Subtitles")
        self.props.short_label = _("Remove")
        self.props.stock_id = gtk.STOCK_REMOVE
        self.props.tooltip = _("Remove the selected subtitles")
        self.accelerator = "Delete"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.view.get_selected_rows())


class SelectAllAction(gaupol.Action):

    """Select all subtitles."""

    def __init__(self):
        """Initialize a :class:`SelectAllAction` object."""
        gaupol.Action.__init__(self, "select_all")
        self.props.label = _("Select _All")
        self.props.stock_id = gtk.STOCK_SELECT_ALL
        self.props.tooltip = _("Select all subtitles")
        self.accelerator = "<Control>A"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.subtitles)


class ShowSelectionMenuAction(gaupol.MenuAction):

    """Show the selection menu."""

    def __init__(self):
        """Initialize a :class:`ShowSelectionMenuAction` object."""
        gaupol.MenuAction.__init__(self, "show_selection_menu")
        self.props.label = _("Sele_ction")
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.subtitles)


class SplitSubtitleAction(gaupol.Action):

    """Split the selected subtitle."""

    def __init__(self):
        """Initialize a :class:`SplitSubtitleAction` object."""
        gaupol.Action.__init__(self, "split_subtitle")
        self.props.label = _("_Split Subtitle")
        self.props.tooltip = _("Split the selected subtitle")
        self.accelerator = "S"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(len(page.view.get_selected_rows()) == 1)


class UndoActionAction(gaupol.Action):

    """Undo the last action."""

    __gtype_name__ = "UndoActionAction"

    def __init__(self):
        """Initialize an :class:`UndoActionAction` object."""
        gaupol.Action.__init__(self, "undo_action")
        self.props.is_important = True
        self.props.label = _("_Undo")
        self.props.stock_id = gtk.STOCK_UNDO
        self.props.tooltip = _("Undo the last action")
        self.accelerator = "<Control>Z"
        self.action_group = "main-unsafe"
        self.set_tool_item_type(gtk.MenuToolButton)

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.can_undo())


__all__ = tuple(filter(lambda x: x.endswith("Action"), dir()))



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Changing the visual appearance of application and its documents."""

import aeidon
import gaupol


class ViewAgent(aeidon.Delegate):

    """Changing the visual appearance of application and its documents."""

    @aeidon.deco.export
    def _on_framerate_combo_changed(self, combo_box):
        """Change the framerate with which nonnative units are calculated."""
        page = self.get_current_page()
        index = combo_box.get_active()
        framerate = aeidon.framerates[index]
        if framerate == page.project.framerate: return
        gaupol.util.set_cursor_busy(self.window)
        page.project.set_framerate(framerate, register=None)
        gaupol.conf.editor.framerate = framerate
        self.get_framerate_action(framerate).set_active(True)
        if page.edit_mode != page.project.main_file.mode:
            rows = range(len(page.project.subtitles))
            fields = filter(lambda x: x.is_position, gaupol.fields)
            page.reload_view(rows, fields)
        gaupol.util.set_cursor_normal(self.window)

    @aeidon.deco.export
    def _on_output_window_notify_visible(self, output_window, visible):
        """Sync menu item to output window's visibility."""
        action = self.get_action("toggle_output_window")
        action.set_active(output_window.props.visible)

    @aeidon.deco.export
    def _on_show_framerate_23_976_changed(self, item, active_item):
        """Change the framerate with which nonnative units are calculated."""
        page = self.get_current_page()
        framerate = active_item.framerate
        if framerate == page.project.framerate: return
        gaupol.util.set_cursor_busy(self.window)
        page.project.set_framerate(framerate, register=None)
        gaupol.conf.editor.framerate = framerate
        self.framerate_combo.set_active(framerate)
        if page.edit_mode != page.project.main_file.mode:
            rows = range(len(page.project.subtitles))
            fields = filter(lambda x: x.is_position, gaupol.fields)
            page.reload_view(rows, fields)
        gaupol.util.set_cursor_normal(self.window)

    @aeidon.deco.export
    def _on_show_times_changed(self, item, active_item):
        """Change the units in which positions are shown."""
        page = self.get_current_page()
        edit_mode = active_item.mode
        if edit_mode == page.edit_mode: return
        gaupol.util.set_cursor_busy(self.window)
        page.edit_mode = edit_mode
        gaupol.conf.editor.mode = edit_mode
        has_focus = page.view.props.has_focus
        focus_row, focus_col = page.view.get_focus()
        selected_rows = page.view.get_selected_rows()
        scroller = page.view.get_parent()
        scroller.remove(page.view)
        page.view = gaupol.View(edit_mode)
        self.connect_view_signals(page.view)
        scroller.add(page.view)
        scroller.show_all()
        page.reload_view_all()
        if focus_row is not None:
            page.view.set_focus(focus_row, focus_col)
            page.view.scroll_to_row(focus_row)
        page.view.select_rows(selected_rows)
        page.view.props.has_focus = has_focus
        gaupol.util.set_cursor_normal(self.window)
        page.emit("view-created", page.view)

    @aeidon.deco.export
    def _on_toggle_duration_column_toggled(self, *args):
        """Show or hide the duration column."""
        self._toggle_column(gaupol.fields.DURATION)

    @aeidon.deco.export
    def _on_toggle_end_column_toggled(self, *args):
        """Show or hide the end column."""
        self._toggle_column(gaupol.fields.END)

    @aeidon.deco.export
    def _on_toggle_main_text_column_toggled(self, *args):
        """Show or hide the main text column."""
        self._toggle_column(gaupol.fields.MAIN_TEXT)

    @aeidon.deco.export
    def _on_toggle_main_toolbar_toggled(self, *args):
        """Show or hide the main toolbar."""
        toolbar = self.uim.get_widget("/ui/main_toolbar")
        visible = toolbar.props.visible
        toolbar.props.visible = not visible
        gaupol.conf.application_window.show_main_toolbar = not visible

    @aeidon.deco.export
    def _on_toggle_number_column_toggled(self, *args):
        """Show or hide the number column."""
        self._toggle_column(gaupol.fields.NUMBER)

    @aeidon.deco.export
    def _on_toggle_output_window_toggled(self, *args):
        """Show or hide the output window."""
        visible = self.output_window.props.visible
        action = self.get_action("toggle_output_window")
        active = action.get_active()
        if visible is active: return
        if self.output_window.props.visible:
            return self.output_window.hide()
        self.output_window.show()

    @aeidon.deco.export
    def _on_toggle_start_column_toggled(self, *args):
        """Show or hide the start column."""
        self._toggle_column(gaupol.fields.START)

    @aeidon.deco.export
    def _on_toggle_statusbar_toggled(self, *args):
        """Show or hide the statusbar."""
        visible = self.statusbar.props.visible
        self.statusbar.props.visible = not visible
        gaupol.conf.application_window.show_statusbar = not visible

    @aeidon.deco.export
    def _on_toggle_translation_text_column_toggled(self, *args):
        """Show or hide the translation text column."""
        self._toggle_column(gaupol.fields.TRAN_TEXT)

    @aeidon.deco.export
    def _on_toggle_video_toolbar_toggled(self, *args):
        """Show or hide the video toolbar."""
        visible = self.video_toolbar.props.visible
        self.video_toolbar.props.visible = not visible
        gaupol.conf.application_window.show_video_toolbar = not visible

    @aeidon.deco.export
    def _on_view_header_button_press_event(self, button, event):
        """Display a column visibility pop-up menu."""
        if event.button != 3: return
        menu = self.uim.get_widget("/ui/view_header_popup")
        menu.popup(None, None, None, event.button, event.time)

    def _toggle_column(self, field):
        """Show or hide column corresponding to `field`."""
        page = self.get_current_page()
        col = getattr(page.view.columns, field.name)
        column = page.view.get_column(col)
        visible = column.get_visible()
        active = self.get_column_action(field).get_active()
        if visible is active: return
        gaupol.util.set_cursor_busy(self.window)
        column.set_visible(not visible)
        visible_fields = []
        for field in gaupol.fields:
            col = getattr(page.view.columns, field.name)
            if page.view.get_column(col).get_visible():
                visible_fields.append(field)
        gaupol.conf.editor.visible_fields = visible_fields
        self.update_gui()
        page.view.columns_autosize()
        gaupol.util.set_cursor_normal(self.window)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Base class for text markup."""

import aeidon
import re

__all__ = ("Markup",)


class Markup(aeidon.Singleton):

    """Base class for text markup.

    Markup conversions between different formats are done via an internal
    format, which has the following BBcode-style tags with angle brackets.
    Conversions are best done via the ``_decode_*`` and ``_encode_*`` methods
    rather than hard-coding the internal tags in regular expression
    substitutions.

     * ``<b>...................</b>``
     * ``<i>...................</i>``
     * ``<u>...................</u>``
     * ``<color=#RRGGBB>...</color>``
     * ``<font=NAME>........</font>``
     * ``<size=POINTS>......</size>``

    This class is effectively equivalent to a format with no markup and can
    therefore be merely subclassed with a ``pass``-statement by formats that do
    not implement any markup. The caller of any tagging methods, e.g.
    :meth:`bolden`, must be prepared to handle :exc:`NotImplementedError`.
    """

    __metaclass__ = aeidon.Contractual
    _flags = re.DOTALL | re.MULTILINE | re.UNICODE
    format = aeidon.formats.NONE

    def _decode_apply_require(self, text, regex, replacement, groups):
        assert replacement.count("%s") == len(groups)

    def _decode_apply(self, text, regex, replacement, groups):
        """Return `text` with all matches of `regex` replaced.

        `replacement` may contain one or more of ``%s``, which are replaced
        with parts of the match as defined `groups`, a ``tuple`` numbers.
        """
        orig_text = text
        match = regex.search(text)
        if match is None: return text
        a, z = match.span()
        new = replacement % tuple(map(match.group, groups))
        text = "".join((text[:a], new, text[z:]))
        if text == orig_text: return text
        return self._decode_apply(text, regex, replacement, groups)

    def _decode_b(self, text, pattern, target, flags=0):
        """Return `text` with bold markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        return self._decode_apply(text, regex, "<b>%s</b>", (target,))

    def _decode_c(self, text, pattern, value, target, flags=0):
        """Return `text` with color markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<color=#%s>%s</color>"
        return self._decode_apply(text, regex, replacement, (value, target))

    def _decode_f(self, text, pattern, value, target, flags=0):
        """Return `text` with font markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<font=%s>%s</font>"
        return self._decode_apply(text, regex, replacement, (value, target))

    def _decode_i(self, text, pattern, target, flags=0):
        """Return `text` with italic markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        return self._decode_apply(text, regex, "<i>%s</i>", (target,))

    def _decode_s(self, text, pattern, value, target, flags=0):
        """Return `text` with size markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<size=%s>%s</size>"
        return self._decode_apply(text, regex, replacement, (value, target))

    def _decode_u(self, text, pattern, target, flags=0):
        """Return `text` with underline markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        return self._decode_apply(text, regex, "<u>%s</u>", (target,))

    def _encode_apply(self, text, regex, method, target, value=None):
        """Return `text` with internal tags removed and `method` applied.

        `method` should be one the tagging methods, e.g. meth:`bolden`.
        `target` and `value` should be group numbers in `regex`.
        """
        orig_text = text
        match = regex.search(text)
        if match is None: return text
        text = regex.sub(r"\%s" % target, text, 1)
        a = match.start()
        z = a + len(match.group(target))
        args = (text, (a, z))
        if value is not None:
            args = (text, match.group(value), (a, z))
        try: text = method(*args)
        except NotImplementedError: pass
        if text == orig_text: return text
        return self._encode_apply(text, regex, method, target, value)

    def _encode_b(self, text):
        """Return `text` with bold markup converted to this format."""
        regex = self._get_regex(r"<b>(.*?)</b>")
        return self._encode_apply(text, regex, self.bolden, 1)

    def _encode_c(self, text):
        """Return `text` with color markup converted to this format."""
        regex = self._get_regex(r"<color=#([a-fA-F0-9]{6})>(.*?)</color>")
        return self._encode_apply(text, regex, self.colorize, 2, 1)

    def _encode_f(self, text):
        """Return `text` with font markup converted to this format."""
        regex = self._get_regex(r"<font=(.+?)>(.*?)</font>")
        return self._encode_apply(text, regex, self.fontify, 2, 1)

    def _encode_i(self, text):
        """Return `text` with italic markup converted to this format."""
        regex = self._get_regex(r"<i>(.*?)</i>")
        return self._encode_apply(text, regex, self.italicize, 1)

    def _encode_s(self, text):
        """Return `text` with size markup converted to this format."""
        regex = self._get_regex(r"<size=(\d+)>(.*?)</size>")
        return self._encode_apply(text, regex, self.scale, 2, 1)

    def _encode_u(self, text):
        """Return `text` with underline markup converted to this format."""
        regex = self._get_regex(r"<u>(.*?)</u>")
        return self._encode_apply(text, regex, self.underline, 1)

    @aeidon.deco.memoize
    def _get_regex(self, pattern, flags=0):
        """Return compiled regular expression from cache."""
        flags = self._flags | flags
        return re.compile(pattern, flags)

    def _main_decode(self, text):
        """Return `text` with decodable markup decoded."""
        return text

    def _post_decode(self, text):
        """Return `text` with markup finalized after decoding."""
        return text

    def _pre_decode(self, text):
        """Return `text` with markup prepared for decoding."""
        return text

    def _substitute(self, text, pattern, replacement, flags=0):
        """Return `text` with matches of `pattern` replaced."""
        regex = self._get_regex(pattern, flags)
        return regex.sub(replacement, text)

    def bolden_require(self, text, bounds=None):
        if bounds is not None:
            assert 0 <= bounds[0] <= bounds[1] <= len(text)

    def bolden(self, text, bounds=None):
        """Return bolded `text`."""
        raise NotImplementedError

    def clean(self, text):
        """Return `text` with less ugly markup.

        Subclasses can implement this to, for example, remove redundant markup,
        finetune tag positioning or to join or split tags, in general, whatever
        that changes the style of the markup but not that of the text.
        """
        return text

    def colorize_require(self, text, color, bounds=None):
        regex = self._get_regex(r"[0-9a-fA-F]{6}")
        assert regex.match(color) is not None
        if bounds is not None:
            assert 0 <= bounds[0] <= bounds[1] <= len(text)

    def colorize(self, text, color, bounds=None):
        """Return `text` colorized to hexadecimal value."""
        raise NotImplementedError

    def decode(self, text):
        """Return `text` with markup converted from this to internal format."""
        text = self._pre_decode(text)
        text = self._main_decode(text)
        return self._post_decode(text)

    def encode(self, text):
        """Return `text` with markup converted from internal to this format."""
        text = self._encode_b(text)
        text = self._encode_c(text)
        text = self._encode_f(text)
        text = self._encode_i(text)
        text = self._encode_s(text)
        return self._encode_u(text)

    def fontify_require(self, text, font, bounds=None):
        if bounds is not None:
            assert 0 <= bounds[0] <= bounds[1] <= len(text)

    def fontify(self, text, font, bounds=None):
        """Return `text` changed to `font`."""
        raise NotImplementedError

    @property
    def italic_tag(self):
        """Regular expression for an italic markup tag or ``None``."""
        return None

    def italicize_require(self, text, bounds=None):
        if bounds is not None:
            assert 0 <= bounds[0] <= bounds[1] <= len(text)

    def italicize(self, text, bounds=None):
        """Return italicized `text`."""
        raise NotImplementedError

    def scale_require(self, text, size, bounds=None):
        assert isinstance(size, int) or size.isdigit()
        if bounds is not None:
            assert 0 <= bounds[0] <= bounds[1] <= len(text)

    def scale(self, text, size, bounds=None):
        """Return `text` scaled to `size`."""
        raise NotImplementedError

    @property
    def tag(self):
        """Regular expression for any markup tag or ``None``."""
        return None

    def underline_require(self, text, bounds=None):
        if bounds is not None:
            assert 0 <= bounds[0] <= bounds[1] <= len(text)

    def underline(self, text, bounds=None):
        """Return underlined `text`."""
        raise NotImplementedError



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Codes, names and descriptions for character encodings.

For functions dealing with character encodings, see also :mod:`aeidon.util`.
"""

import aeidon
import codecs
import locale
import re
_ = aeidon.i18n._

# Tuples of code, name and description for each supported character encoding.
# Codes are the official names used by Python. Names are mostly taken from
# IANA. Descriptions are mostly copied from gedit.

_encodings = (
    # Translators: Most of the character encoding descriptions are copied from
    # gedit, which is translated to very many languages. Check the gedit .po
    # files for a reference: <http://git.gnome.org/cgit/gedit/tree/po/>.
    ("ascii"          , "US-ASCII"        , _("English")            ),
    ("big5"           , "Big5"            , _("Chinese traditional")),
    ("big5hkscs"      , "Big5-HKSCS"      , _("Chinese traditional")),
    ("cp037"          , "IBM037"          , _("English")            ),
    ("cp424"          , "IBM242"          , _("Hebrew")             ),
    ("cp437"          , "IBM437"          , _("English")            ),
    ("cp500"          , "IBM500"          , _("Western")            ),
    ("cp737"          , "IBM737"          , _("Greek")              ),
    ("cp775"          , "IBM775"          , _("Baltic")             ),
    ("cp850"          , "IBM850"          , _("Western")            ),
    ("cp852"          , "IBM852"          , _("Central European")   ),
    ("cp855"          , "IBM855"          , _("Cyrillic")           ),
    ("cp856"          , "IBM856"          , _("Hebrew")             ),
    ("cp857"          , "IBM857"          , _("Turkish")            ),
    ("cp860"          , "IBM860"          , _("Portuguese")         ),
    ("cp861"          , "IBM861"          , _("Icelandic")          ),
    ("cp862"          , "IBM862"          , _("Hebrew")             ),
    ("cp863"          , "IBM863"          , _("Canadian")           ),
    ("cp864"          , "IBM864"          , _("Arabic")             ),
    ("cp865"          , "IBM865"          , _("Nordic")             ),
    ("cp866"          , "IBM866"          , _("Russian")            ),
    ("cp869"          , "IBM869"          , _("Greek")              ),
    ("cp874"          , "IBM874"          , _("Thai")               ),
    ("cp875"          , "IBM875"          , _("Greek")              ),
    ("cp932"          , "IBM932"          , _("Japanese")           ),
    ("cp949"          , "IBM949"          , _("Korean")             ),
    ("cp950"          , "IBM950"          , _("Chinese traditional")),
    ("cp1006"         , "IBM1006"         , _("Urdu")               ),
    ("cp1026"         , "IBM1026"         , _("Turkish")            ),
    ("cp1140"         , "IBM1140"         , _("Western")            ),
    ("cp1250"         , "windows-1250"    , _("Central European")   ),
    ("cp1251"         , "windows-1251"    , _("Cyrillic")           ),
    ("cp1252"         , "windows-1252"    , _("Western")            ),
    ("cp1253"         , "windows-1253"    , _("Greek")              ),
    ("cp1254"         , "windows-1254"    , _("Turkish")            ),
    ("cp1255"         , "windows-1255"    , _("Hebrew")             ),
    ("cp1256"         , "windows-1256"    , _("Arabic")             ),
    ("cp1257"         , "windows-1257"    , _("Baltic")             ),
    ("cp1258"         , "windows-1258"    , _("Vietnamese")         ),
    ("euc_jp"         , "EUC-JP"          , _("Japanese")           ),
    ("euc_jis_2004"   , "EUC-JIS-2004"    , _("Japanese")           ),
    ("euc_jisx0213"   , "EUC-JISX0213"    , _("Japanese")           ),
    ("euc_kr"         , "EUC-KR"          , _("Korean")             ),
    ("gb2312"         , "GB2312"          , _("Chinese simplified") ),
    ("gbk"            , "GBK"             , _("Chinese unified")    ),
    ("gb18030"        , "GB18030"         , _("Chinese unified")    ),
    ("hz"             , "HZ"              , _("Chinese simplified") ),
    ("iso2022_jp"     , "ISO-2022-JP"     , _("Japanese")           ),
    ("iso2022_jp_1"   , "ISO-2022-JP-1"   , _("Japanese")           ),
    ("iso2022_jp_2"   , "ISO-2022-JP-2"   , _("Japanese")           ),
    ("iso2022_jp_2004", "ISO-2022-JP-2004", _("Japanese")           ),
    ("iso2022_jp_3"   , "ISO-2022-JP-3"   , _("Japanese")           ),
    ("iso2022_jp_ext" , "ISO-2022-JP-EXT" , _("Japanese")           ),
    ("iso2022_kr"     , "ISO-2022-KR"     , _("Korean")             ),
    ("latin_1"        , "ISO-8859-1"      , _("Western")            ),
    ("iso8859_2"      , "ISO-8859-2"      , _("Central European")   ),
    ("iso8859_3"      , "ISO-8859-3"      , _("South European")     ),
    ("iso8859_4"      , "ISO-8859-4"      , _("Baltic")             ),
    ("iso8859_5"      , "ISO-8859-5"      , _("Cyrillic")           ),
    ("iso8859_6"      , "ISO-8859-6"      , _("Arabic")             ),
    ("iso8859_7"      , "ISO-8859-7"      , _("Greek")              ),
    ("iso8859_8"      , "ISO-8859-8"      , _("Hebrew")             ),
    ("iso8859_9"      , "ISO-8859-9"      , _("Turkish")            ),
    ("iso8859_10"     , "ISO-8859-10"     , _("Nordic")             ),
    ("iso8859_13"     , "ISO-8859-13"     , _("Baltic")             ),
    ("iso8859_14"     , "ISO-8859-14"     , _("Celtic")             ),
    ("iso8859_15"     , "ISO-8859-15"     , _("Western")            ),
    ("johab"          , "Johab"           , _("Korean")             ),
    ("koi8_r"         , "KOI8-R"          , _("Russian")            ),
    ("koi8_u"         , "KOI8-U"          , _("Ukrainian")          ),
    ("mac_cyrillic"   , "MacCyrillic"     , _("Cyrillic")           ),
    ("mac_greek"      , "MacGreek"        , _("Greek")              ),
    ("mac_iceland"    , "MacIceland"      , _("Icelandic")          ),
    ("mac_latin2"     , "MacCentralEurope", _("Central European")   ),
    ("mac_roman"      , "MacRoman"        , _("Western")            ),
    ("mac_turkish"    , "MacTurkish"      , _("Turkish")            ),
    ("ptcp154"        , "PTCP154"         , _("Cyrillic Asian")     ),
    ("shift_jis"      , "Shift_JIS"       , _("Japanese")           ),
    ("shift_jis_2004" , "Shift_JIS-2004"  , _("Japanese")           ),
    ("shift_jisx0213" , "Shift_JISX0213"  , _("Japanese")           ),
    ("utf_16"         , "UTF-16"          , _("Unicode")            ),
    ("utf_16_be"      , "UTF-16BE"        , _("Unicode")            ),
    ("utf_16_le"      , "UTF-16LE"        , _("Unicode")            ),
    ("utf_7"          , "UTF-7"           , _("Unicode")            ),
    ("utf_8"          , "UTF-8"           , _("Unicode")            ),
    ("utf_8_sig"      , "UTF-8-SIG"       , _("Unicode")            ),)

CODE, NAME, DESC = range(3)

# Characters illegal in encoding codes.
_re_illegal = re.compile(r"[^a-z0-9_]")


def code_to_description(code):
    """Convert encoding `code` to localized description.

    Raise :exc:`ValueError` if not found.

    >>> aeidon.encodings.code_to_description("utf_8")
    'Unicode'
    """
    for item in _encodings:
        if item[CODE] == code:
            return item[DESC]
    raise ValueError("Code %s not found" % repr(code))

def code_to_long_name(code):
    """Convert encoding `code` to localized long name.

    Raise :exc:`ValueError` if not found.
    Return localized ``DESCRIPTION (DISPLAY NAME)``.

    >>> aeidon.encodings.code_to_long_name("utf_8")
    'Unicode (UTF-8)'
    """
    for item in _encodings:
        if item[CODE] == code:
            name, description = item[NAME], item[DESC]
            return _("%(description)s (%(name)s)") % locals()
    raise ValueError("Code %s not found" % repr(code))

def code_to_name(code):
    """Convert encoding `code` to name.

    Raise :exc:`ValueError` if not found.

    >>> aeidon.encodings.code_to_name("utf_8")
    'UTF-8'
    """
    for item in _encodings:
        if item[CODE] == code:
            return item[NAME]
    raise ValueError("Code %s not found" % repr(code))

def detect_ensure(value, path):
    if value is not None:
        assert is_valid_code(value)

@aeidon.deco.contractual
def detect(path):
    """Detect the encoding of file at `path` and return code or ``None``.

    Raise :exc:`IOError` if reading fails.
    """
    from chardet import universaldetector
    detector = universaldetector.UniversalDetector()
    with open(path, "r") as fobj:
        for line in fobj:
            detector.feed(line)
            if detector.done: break
    detector.close()
    code = detector.result["encoding"]
    if code is None: return None
    try:
        # chardet returns what seem to be IANA names. They need to be
        # translated to their Python equivalents. Some of the encodings
        # returned by chardet are not supported by Python.
        return translate_code(code)
    except ValueError:
        return None

def detect_bom_ensure(value, path):
    if value is not None:
        assert is_valid_code(value)

@aeidon.deco.contractual
def detect_bom(path):
    """Return corresponding encoding if BOM found, else ``None``."""
    line = open(path, "r").readline()
    if (line.startswith(codecs.BOM_UTF32_BE) and
        is_valid_code("utf_32_be")):
        return "utf_32_be"
    if (line.startswith(codecs.BOM_UTF32_LE) and
        is_valid_code("utf_32_le")):
        return "utf_32_le"
    if (line.startswith(codecs.BOM_UTF8) and
        is_valid_code("utf_8_sig")):
        return "utf_8_sig"
    if (line.startswith(codecs.BOM_UTF16_BE) and
        is_valid_code("utf_16_be")):
        return "utf_16_be"
    if (line.startswith(codecs.BOM_UTF16_LE) and
        is_valid_code("utf_16_le")):
        return "utf_16_le"
    return None

def get_locale_code_ensure(value):
    if value is not None:
        assert is_valid_code(value)

@aeidon.deco.once
@aeidon.deco.contractual
def get_locale_code():
    """Return code of the locale encoding or ``None``."""
    code = locale.getpreferredencoding()
    if code is None: return None
    code = _re_illegal.sub("_", code.lower())
    code = aeidon.util.get_encoding_alias(code)
    return code

def get_locale_long_name_require():
    assert get_locale_code() is not None

@aeidon.deco.once
@aeidon.deco.contractual
def get_locale_long_name():
    """Return localized long name for locale encoding.

    Raise :exc:`ValueError` if not found.
    Return localized ``CURRENT LOCALE (NAME)``.
    """
    name = code_to_name(get_locale_code())
    return _("Current locale (%s)") % name

@aeidon.deco.once
def get_valid():
    """Return a sequence of valid encodings.

    Return a tuple of tuples of code, name, description.
    """
    valid_encodings = []
    for i, item in enumerate(_encodings):
        if is_valid_code(item[CODE]):
            valid_encodings.append(item)
    return tuple(valid_encodings)

def is_valid_code(code):
    """Return ``True`` if encoding `code` is valid."""
    try:
        codecs.lookup(code)
    except LookupError:
        return False
    return True

def name_to_code(name):
    """Convert encoding `name` to code.

    Raise :exc:`ValueError` if not found.

    >>> aeidon.encodings.name_to_code("UTF-8")
    'utf_8'
    """
    for item in _encodings:
        if item[NAME] == name:
            return item[CODE]
    raise ValueError("Name %s not found" % repr(name))

def translate_code_ensure(value, code):
    assert is_valid_code(value)

@aeidon.deco.contractual
def translate_code(code):
    """Translate weird encoding `code`.

    Raise :exc:`ValueError` if not found.
    Return normalized encoding code.

    >>> aeidon.encodings.translate_code("ISO-8859-1")
    'latin_1'
    """
    code = _re_illegal.sub("_", code.lower())
    code = aeidon.util.get_encoding_alias(code)
    for item in _encodings:
        if item[CODE] == code:
            return item[CODE]
    raise ValueError("Code %s not found" % repr(code))



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for converting framerates."""

import aeidon
import gaupol
import gtk

__all__ = ("FramerateConvertDialog",)


class FramerateConvertDialog(gaupol.BuilderDialog):

    """Dialog for converting framerates."""

    _widgets = ("all_radio", "current_radio", "input_combo", "output_combo")

    def __init__(self, parent, application):
        """Initialize a FramerateConvertDialog object."""
        gaupol.BuilderDialog.__init__(self, "framerate-dialog.ui")
        self.application = application
        self._init_input_combo()
        self._init_output_combo()
        self._init_values()
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_OK)

    def _convert_framerates(self):
        """Convert framerates in target projects."""
        target = self._get_target()
        framerate_in = aeidon.framerates[self._input_combo.get_active()]
        framerate_out = aeidon.framerates[self._output_combo.get_active()]
        for page in self.application.get_target_pages(target):
            self.application.set_current_page(page)
            page.project.convert_framerate(None, framerate_in, framerate_out)

    def _get_target(self):
        """Return the selected target."""
        if self._current_radio.get_active():
            return gaupol.targets.CURRENT
        if self._all_radio.get_active():
            return gaupol.targets.ALL
        raise ValueError("Invalid target radio state")

    def _init_input_combo(self):
        """Initialize the input framerate combo box."""
        store = gtk.ListStore(str)
        self._input_combo.set_model(store)
        for label in (x.label for x in aeidon.framerates):
            store.append((label,))
        renderer = gtk.CellRendererText()
        self._input_combo.pack_start(renderer, True)
        self._input_combo.add_attribute(renderer, "text", 0)

    def _init_output_combo(self):
        """Initialize the output framerate combo box."""
        store = gtk.ListStore(str)
        self._output_combo.set_model(store)
        for label in (x.label for x in aeidon.framerates):
            store.append((label,))
        renderer = gtk.CellRendererText()
        self._output_combo.pack_start(renderer, True)
        self._output_combo.add_attribute(renderer, "text", 0)

    def _init_values(self):
        """Intialize default values for widgets."""
        page = self.application.get_current_page()
        self._input_combo.set_active(page.project.framerate)
        self._output_combo.set_active(page.project.framerate)
        target = gaupol.conf.framerate_convert.target
        self._all_radio.set_active(target == gaupol.targets.ALL)
        self._current_radio.set_active(target == gaupol.targets.CURRENT)

    def _on_input_combo_changed(self, *args):
        """Set response sensitivity."""
        index_in = self._input_combo.get_active()
        index_out = self._output_combo.get_active()
        sensitive = (index_in != index_out)
        self._dialog.set_response_sensitive(gtk.RESPONSE_OK, sensitive)

    def _on_output_combo_changed(self, *args):
        """Set response sensitivity."""
        index_in = self._input_combo.get_active()
        index_out = self._output_combo.get_active()
        sensitive = (index_in != index_out)
        self._dialog.set_response_sensitive(gtk.RESPONSE_OK, sensitive)

    def _on_response(self, dialog, response):
        """Save target and convert framerates."""
        gaupol.conf.framerate_convert.target = self._get_target()
        if response == gtk.RESPONSE_OK:
            self._convert_framerates()



#!/usr/bin/env python

"""py2exe installation routines built on top of normal setup.py."""

# pylint: disable=W0614
from setup import *

includes = ("aeidon",
            "atk",
            "cairo",
            "chardet",
            "enchant",
            "gaupol",
            "gio",
            "glib",
            "gobject",
            "gtk",
            "pango",
            "pangocairo",
            "pygtk",
            )

setup_kwargs.update(dict(
    windows=[dict(script="bin/gaupol",
                  icon_resources=[(0, "data/icons/gaupol.ico")])],

    options=dict(py2exe=dict(includes=list(includes),
                             packages=list(includes)))))

if __name__ == "__main__":
    distutils.core.setup(**setup_kwargs)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol
import gtk


class TestLanguageDialog(gaupol.TestCase):

    def run__dialog__hide_target(self):
        # pylint: disable=W0201
        self.dialog.destroy()
        self.dialog = gaupol.LanguageDialog(gtk.Window(), False)
        self.dialog.run()
        self.dialog.destroy()

    def run__dialog__show_target(self):
        # pylint: disable=W0201
        self.dialog.destroy()
        self.dialog = gaupol.LanguageDialog(gtk.Window(), True)
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        self.dialog = gaupol.LanguageDialog(gtk.Window())
        self.dialog.show()

    def test___init__(self):
        gaupol.conf.spell_check.language = "en"
        gaupol.LanguageDialog(gtk.Window(), True)
        gaupol.LanguageDialog(gtk.Window(), False)

    def test__on_all_radio_toggled(self):
        self.dialog._all_radio.set_active(True)
        self.dialog._all_radio.set_active(False)

    def test__on_current_radio_toggled(self):
        self.dialog._current_radio.set_active(True)
        self.dialog._current_radio.set_active(False)

    def test__on_main_radio_toggled(self):
        self.dialog._main_radio.set_active(True)
        self.dialog._main_radio.set_active(False)

    def test__on_tran_radio_toggled(self):
        self.dialog._tran_radio.set_active(True)
        self.dialog._tran_radio.set_active(False)

    def test__on_tree_view_selection_changed(self):
        store = self.dialog._tree_view.get_model()
        selection = self.dialog._tree_view.get_selection()
        store = self.dialog._tree_view.get_model()
        for i in range(len(store)):
            selection.select_path(i)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import codecs
import os


class TestOpenAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()
        self.delegate = self.project.open_main.im_self

    def test_open__main(self):
        for format in aeidon.formats:
            self.project.remove_subtitles((0,))
            self.project.open(aeidon.documents.MAIN,
                              self.new_temp_file(format),
                              "ascii")

            assert self.project.subtitles

    def test_open__translation(self):
        for format in aeidon.formats:
            self.project.remove_subtitles((0,))
            self.project.open(aeidon.documents.TRAN,
                              self.new_temp_file(format),
                              "ascii")

    def test_open_main(self):
        for format in aeidon.formats:
            self.project.remove_subtitles((0,))
            self.project.open_main(self.new_temp_file(format),
                                   "ascii")

            assert self.project.subtitles

    def test_open_main__bom(self):
        path = self.new_subrip_file()
        text = open(path, "r").read()
        open(path, "w").write(codecs.BOM_UTF8 + text)
        self.project.open_main(path, "ascii")
        assert self.project.subtitles
        assert self.project.main_file.encoding == "utf_8_sig"

    def test_open_main__mpsub(self):
        self.project.remove_subtitles((0,))
        path = self.new_temp_file(aeidon.formats.MPSUB, "mpsub-frame")
        self.project.open_main(path, "ascii")
        assert self.project.subtitles

    def test_open_main__io_error(self):
        path = self.new_subrip_file()
        os.chmod(path, 0000)
        self.raises(IOError,
                    self.project.open_main,
                    path, "ascii")

        os.chmod(path, 0777)

    @aeidon.deco.monkey_patch(aeidon, "debug")
    def test_open_main__parse_error(self):
        aeidon.debug = False
        path = self.new_subrip_file()
        fobj = open(path, "w")
        fobj.write("00:00:01,000 --> 00:00:02,000\n\n")
        fobj.write("00:00:03,000 <-- 00:00:04,000\n\n")
        fobj.close()
        self.raises(aeidon.ParseError,
                    self.project.open_main,
                    path, "ascii")

    def test_open_main__unicode_error(self):
        self.raises(UnicodeError,
                    self.project.open_main,
                    self.new_subrip_file(), "punycode")

    def test_open_main__unsorted(self):
        path = self.new_microdvd_file()
        fobj = open(path, "w")
        fobj.write("{100}{200}\n")
        fobj.write("{500}{600}\n")
        fobj.write("{300}{400}\n")
        fobj.close()
        assert self.project.open_main(path, "ascii") == 1

    def test_open_translation__align_number(self):
        for format in aeidon.formats:
            self.project.remove_subtitles((0,))
            self.project.open_translation(self.new_temp_file(format),
                                          "ascii",
                                          aeidon.align_methods.NUMBER)

    def test_open_translation__align_position(self):
        for format in aeidon.formats:
            self.project.remove_subtitles((0,))
            self.project.open_translation(self.new_temp_file(format),
                                          "ascii",
                                          aeidon.align_methods.POSITION)

    def test_open_translation__bom(self):
        path = self.new_subrip_file()
        text = open(path, "r").read()
        open(path, "w").write(codecs.BOM_UTF8 + text)
        self.project.open_translation(path, "ascii")
        assert self.project.tran_file.encoding == "utf_8_sig"



# Copyright (C) 2005-2007,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Window for standard output from external applications."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._

__all__ = ("OutputWindow",)


class OutputWindow(gtk.Window):

    """Window for standard output from external applications."""

    def __init__(self):
        """Initialize an :class:`OutputWindow` object."""
        gtk.Window.__init__(self)
        self._close_button = None
        self._text_view = None
        self.set_border_width(12)
        self.set_title(_("Output"))
        self._init_widgets()
        self._init_sizes()
        self._init_signal_handlers()
        self._init_keys()

    def _init_keys(self):
        """Initialize keyboard shortcuts."""
        accel_group = gtk.AccelGroup()
        accel_group.connect_group(gtk.keysyms.w,
                                  gtk.gdk.CONTROL_MASK,
                                  gtk.ACCEL_MASK,
                                  self._on_close_key_pressed)

        self.add_accel_group(accel_group)

    def _init_signal_handlers(self):
        """Initialize signal handlers."""
        aeidon.util.connect(self, "_close_button", "clicked")
        aeidon.util.connect(self, self, "delete-event")
        aeidon.util.connect(self, self, "notify::visible")
        aeidon.util.connect(self, self, "window-state-event")

    def _init_sizes(self):
        """Initialize widget sizes."""
        self.resize(*gaupol.conf.output_window.size)
        self.move(*gaupol.conf.output_window.position)
        if gaupol.conf.output_window.maximized:
            self.maximize()

    def _init_widgets(self):
        """Initialize all contained widgets."""
        self._text_view = gtk.TextView()
        self._text_view.set_wrap_mode(gtk.WRAP_WORD)
        self._text_view.set_cursor_visible(False)
        self._text_view.set_editable(False)
        self._text_view.set_left_margin(6)
        self._text_view.set_right_margin(6)
        self._text_view.set_pixels_below_lines(1)
        gaupol.util.set_widget_font(self._text_view, "monospace")
        scroller = gtk.ScrolledWindow()
        scroller.set_policy(*((gtk.POLICY_AUTOMATIC,) * 2))
        scroller.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        scroller.add(self._text_view)
        self._close_button = gtk.Button(stock=gtk.STOCK_CLOSE)
        button_box = gtk.HButtonBox()
        button_box.set_layout(gtk.BUTTONBOX_END)
        button_box.pack_start(self._close_button, False, False)
        vbox = gtk.VBox(spacing=12)
        vbox.pack_start(scroller, True, True)
        vbox.pack_start(button_box, False, False)
        vbox.show_all()
        self.add(vbox)

    def _on_close_button_clicked(self, *args):
        """Hide window."""
        self._save_geometry()
        self.hide()

    def _on_close_key_pressed(self, *args):
        """Hide window."""
        self._save_geometry()
        self.hide()

    def _on_delete_event(self, *args):
        """Hide window."""
        self._save_geometry()
        self.hide()
        return True

    def _on_notify_visible(self, *args):
        """Save window visibility."""
        gaupol.conf.output_window.show = self.props.visible

    def _on_window_state_event(self, window, event):
        """Save window maximization."""
        state = event.new_window_state
        maximized = bool(state & gtk.gdk.WINDOW_STATE_MAXIMIZED)
        gaupol.conf.output_window.maximized = maximized

    def _save_geometry(self):
        """Save window size and position."""
        if gaupol.conf.output_window.maximized: return
        gaupol.conf.output_window.size = list(self.get_size())
        gaupol.conf.output_window.position = list(self.get_position())

    def set_output(self, output):
        """Display `output` in text view."""
        text_buffer = self._text_view.get_buffer()
        text_buffer.set_text(output)
        mark = text_buffer.get_insert()
        self._text_view.scroll_to_mark(mark, 0)



# Copyright (C) 2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import os
import shutil


class TestConfigurationStore(gaupol.TestCase):

    # pylint: disable=E1101

    def setup_method(self, method):
        self.conf = gaupol.ConfigurationStore()
        self.directory = aeidon.temp.create_directory()
        self.conf.path = os.path.join(self.directory, "test")

    def teardown_method(self, method):
        shutil.rmtree(self.directory)

    def test_connect_notify(self):
        class PuppetObserver(object):
            def _on_conf_application_window_notify_size(self, obj, value):
                assert value == [99, 99]
        puppet = PuppetObserver()
        self.conf.connect_notify("application_window", "size", puppet)
        self.conf.application_window.size = [99, 99]

    def test_disconnect_notify(self):
        class PuppetObserver(object):
            def _on_conf_application_window_notify_size(self, obj, value):
                assert value == [99, 99]
        puppet = PuppetObserver()
        self.conf.connect_notify("application_window", "size", puppet)
        self.conf.application_window.size = [99, 99]
        self.conf.disconnect_notify("application_window", "size", puppet)
        self.conf.application_window.size = [100, 100]

    def test_query_default(self):
        assert not self.conf.query_default("application_window", "maximized")
        self.conf.application_window.maximized = True
        assert not self.conf.query_default("application_window", "maximized")

    def test_read_from_file(self):
        self.conf.write_to_file()
        self.conf.restore_defaults()
        self.conf.read_from_file()

    def test_register_extension(self):
        self.conf.register_extension("test",
                                     {"x": 1, "mode": aeidon.modes.TIME},
                                     {"mode": aeidon.modes})

        assert self.conf.extensions.test.x == 1
        assert self.conf.extensions.test.mode == aeidon.modes.TIME

    def test_restore_defaults(self):
        size = list(self.conf.application_window.size)
        self.conf.application_window.size = [99, 99]
        self.conf.restore_defaults()
        assert self.conf.application_window.size == size

    def test_write_to_file(self):
        self.conf.write_to_file()
        self.conf.restore_defaults()
        self.conf.read_from_file()

    def test_write_to_file__io_error(self):
        os.chmod(self.directory, 0000)
        self.raises(IOError, self.conf.write_to_file)
        os.chmod(self.directory, 0777)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Subtitle files of all formats."""

import aeidon

aeidon.util.install_module("files", lambda: None)

from .ssa        import SubStationAlpha
from .ass        import AdvSubStationAlpha
from .microdvd   import MicroDVD
from .mpl2       import MPL2
from .mpsub      import MPsub
from .subrip     import SubRip
from .subviewer2 import SubViewer2
from .tmplayer   import TMPlayer

__all__ = aeidon.util.get_all(dir(), r"^[A-Z]")


def add(cls):
    """Add a new :class:`aeidon.SubtitleFile` class."""
    globals()[cls.__name__] = cls
    globals()["__all__"] = tuple(set(__all__ + (cls.__name__,)))

def new(format, path, encoding, newline=None):
    """Return a new :class:`aeidon.SubtitleFile` instance given `format`."""
    for cls in map(eval, __all__):
        if cls.format == format:
            return cls(path, encoding, newline)
    raise ValueError("Format %s not found" % repr(format))



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestMetadataItem(aeidon.TestCase):

    @aeidon.deco.monkey_patch(aeidon.locales, "get_system_code")
    @aeidon.deco.monkey_patch(aeidon.locales, "get_system_modifier")
    def assert_name_in_locale(self, code, modifier=None):
        aeidon.locales.get_system_code = lambda: code
        aeidon.locales.get_system_modifier = lambda: modifier
        self.item.set_field("Name", "system")
        key = (code if modifier is None else "%s@%s" % (code, modifier))
        self.item.set_field("Name[%s]" % key, "local")
        assert self.item.get_name() == "local"

    def setup_method(self, method):
        self.item = aeidon.MetadataItem()

    def test_get_description(self):
        self.item.set_field("Description", "test")
        assert self.item.get_description(False) == "test"
        assert self.item.get_description(True) == "test"

    def test_get_field(self):
        assert self.item.get_field("Xxxx") is None
        self.item.set_field("Test", "test")
        assert self.item.get_field("Test") == "test"

    def test_get_field_boolean(self):
        assert self.item.get_field("Xxxx") is None
        self.item.set_field("Test", "True")
        assert self.item.get_field_boolean("Test") is True
        self.item.set_field("Test", "False")
        assert self.item.get_field_boolean("Test") is False

    def test_get_field_boolean__fallback(self):
        assert self.item.get_field("Xxxx", False) is False

    def test_get_field_boolean__value_error(self):
        self.item.set_field("Test", "Xxxx")
        function = self.item.get_field_boolean
        self.raises(ValueError, function, "Test")

    def test_get_field_list(self):
        assert self.item.get_field("Xxxx") is None
        self.item.set_field("Test", "Yee;Haw")
        assert self.item.get_field_list("Test") == ["Yee", "Haw"]
        self.item.set_field("Test", "Yee;Haw;")
        assert self.item.get_field_list("Test") == ["Yee", "Haw"]

    def test_get_field_list__fallback(self):
        assert self.item.get_field("Xxxx", []) == []

    def test_get_name(self):
        self.item.set_field("Name", "test")
        assert self.item.get_name(False) == "test"
        assert self.item.get_name(True) == "test"

    def test_get_name__localize(self):
        self.assert_name_in_locale("en_US", "Latn")
        self.assert_name_in_locale("en_US", None)
        self.assert_name_in_locale("en", "Latn")
        self.assert_name_in_locale("en", None)

    @aeidon.deco.monkey_patch(aeidon.locales, "get_system_code")
    def test_get_name__no_locale(self):
        aeidon.locales.get_system_code = lambda: None
        self.item.set_field("Name", "system")
        assert self.item.get_name() == "system"

    def test_has_field(self):
        assert not self.item.has_field("Test")
        self.item.set_field("Test", "")
        assert self.item.has_field("Test")

    def test_set_field(self):
        self.item.set_field("Test", "test")
        assert self.item.get_field("Test") == "test"



# Copyright (C) 2005-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestEditAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()

    @aeidon.deco.reversion_test
    def test_clear_texts(self):
        self.project.clear_texts((0, 1), aeidon.documents.MAIN)
        assert self.project.subtitles[0].main_text == ""
        assert self.project.subtitles[1].main_text == ""

    @aeidon.deco.reversion_test
    def test_insert_subtitles(self):
        subtitles = self.project.subtitles
        orig_length = len(subtitles)
        new_subtitles = []
        for i in range(3):
            subtitle = self.project.new_subtitle()
            subtitle.start = i
            subtitle.end = i + 1
            subtitle.main_text = str(i)
            subtitle.tran_text = str(i)
            new_subtitles.append(subtitle)
        self.project.insert_subtitles((0, 1, 2), new_subtitles)
        assert len(subtitles) == orig_length + 3
        assert subtitles[0:3] == new_subtitles

    @aeidon.deco.reversion_test
    def test_insert_subtitles__blank_end(self):
        subtitles = self.project.subtitles
        orig_length = len(subtitles)
        indices = range(orig_length, orig_length + 10)
        self.project.insert_subtitles(indices)
        assert len(subtitles) == orig_length + 10
        assert subtitles == sorted(subtitles)

    @aeidon.deco.reversion_test
    def test_insert_subtitles__blank_middle(self):
        subtitles = self.project.subtitles
        orig_length = len(subtitles)
        self.project.insert_subtitles((1, 2, 3))
        assert len(subtitles) == orig_length + 3
        assert subtitles == sorted(subtitles)

    @aeidon.deco.reversion_test
    def test_insert_subtitles__blank_start(self):
        subtitles = self.project.subtitles
        orig_length = len(subtitles)
        self.project.insert_subtitles((0, 1))
        assert len(subtitles) == orig_length + 2
        assert subtitles == sorted(subtitles)

    @aeidon.deco.reversion_test
    def test_merge_subtitles(self):
        subtitles = self.project.subtitles
        subtitle_1 = subtitles[1].copy()
        subtitle_2 = subtitles[2].copy()
        orig_length = len(subtitles)
        self.project.merge_subtitles((1, 2))
        assert len(subtitles) == orig_length - 1
        assert subtitles[1].start == subtitle_1.start
        assert subtitles[1].end == subtitle_2.end

    @aeidon.deco.reversion_test
    def test_remove_subtitles(self):
        subtitles = self.project.subtitles
        orig_length = len(subtitles)
        self.project.remove_subtitles((2, 3))
        assert len(subtitles) == orig_length - 2

    @aeidon.deco.reversion_test
    def test_replace_positions(self):
        new_subtitles = []
        for i in range(3):
            subtitle = self.project.new_subtitle()
            subtitle.start = i
            subtitle.end = i + 1
            new_subtitles.append(subtitle)
        self.project.replace_positions((0, 1, 2), new_subtitles)
        subtitles = self.project.subtitles
        for i in range(3):
            assert subtitles[i].start == new_subtitles[i].start
            assert subtitles[i].end == new_subtitles[i].end

    @aeidon.deco.reversion_test
    def test_replace_texts(self):
        doc = aeidon.documents.MAIN
        self.project.replace_texts((1, 2), doc, ("", ""))
        assert self.project.subtitles[1].main_text == ""
        assert self.project.subtitles[2].main_text == ""

    @aeidon.deco.reversion_test
    def test_split_subtitle(self):
        subtitles = self.project.subtitles
        subtitle = subtitles[1].copy()
        orig_length = len(subtitles)
        self.project.split_subtitle(1)
        assert len(subtitles) == orig_length + 1
        assert subtitles[1].start == subtitle.start
        assert subtitles[1].end < subtitle.end
        assert subtitles[2].start < subtitle.end
        assert subtitles[2].end == subtitle.end



# Copyright (C) 2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestExtensionManager(gaupol.TestCase):

    def setup_method(self, method):
        gaupol.conf.path = aeidon.temp.create(".conf")
        self.manager = gaupol.ExtensionManager(self.new_application())
        gaupol.conf.extensions.active = ["bookmarks", "none"]
        self.manager.find_extensions()
        self.manager.setup_extensions()

    def teardown_method(self, method):
        self.manager.teardown_extensions()
        gaupol.TestCase.teardown_method(self, method)

    def test_find_extensions(self):
        self.manager.find_extensions()
        assert self.manager.get_modules()

    def test_get_metadata(self):
        metadata = self.manager.get_metadata("bookmarks")
        assert metadata.has_field("GaupolVersion")
        assert metadata.has_field("Module")
        assert metadata.has_field("Name")
        assert metadata.has_field("Description")

    def test_get_modules(self):
        modules = self.manager.get_modules()
        assert "bookmarks" in modules

    def test_has_help(self):
        self.manager.has_help("bookmarks")

    def test_has_preferences_dialog(self):
        self.manager.has_preferences_dialog("bookmarks")

    def test_is_active(self):
        self.manager.setup_extension("bookmarks")
        assert self.manager.is_active("bookmarks")
        self.manager.teardown_extension("bookmarks")
        assert not self.manager.is_active("bookmarks")

    def test_setup_extension(self):
        self.manager.setup_extension("bookmarks")

    def test_setup_extensions(self):
        self.manager.setup_extensions()

    def test_show_help(self):
        try: self.manager.show_help("bookmarks")
        except NotImplementedError: pass

    def test_show_preferences_dialog(self):
        try: self.manager.show_preferences_dialog("bookmarks", gtk.Window())
        except NotImplementedError: pass

    def test_teardown_extension(self):
        self.manager.setup_extension("bookmarks")
        self.manager.teardown_extension("bookmarks")

    def test_teardown_extensions(self):
        self.manager.teardown_extensions()

    def test_update_extensions(self):
        page = self.manager.application.get_current_page()
        self.manager.update_extensions(page)



# Copyright (C) 2005-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Manipulating times and frames."""

from __future__ import division

import aeidon
_ = aeidon.i18n._


class PositionAgent(aeidon.Delegate):

    """Manipulating times and frames."""

    # pylint: disable=E0203,E1101,W0201

    __metaclass__ = aeidon.Contractual

    def _get_frame_transform_ensure(self, value, p1, p2):
        assert isinstance(value[1], int)

    def _get_frame_transform(self, p1, p2):
        """Return a formula for linear transformation of positions.

        Return coefficient and constant, with which all subtitles
        should be scaled and shifted by to apply correction.
        """
        # Think of this as a linear transformation where input positions
        # are located on the x-axis and output positions on the y-axis.
        x1 = self.subtitles[p1[0]].start_frame
        x2 = self.subtitles[p2[0]].start_frame
        y1 = p1[1]
        y2 = p2[1]
        coefficient = (y2 - y1) / (x2 - x1)
        constant = int(round(- (coefficient * x1) + y1, 0))
        return coefficient, constant

    def _get_seconds_transform_ensure(self, value, p1, p2):
        assert isinstance(value[1], float)

    def _get_seconds_transform(self, p1, p2):
        """Return a formula for linear transformation of positions.

        Return coefficient and constant, with which all subtitles should be
        scaled and shifted by to apply correction.
        """
        # Think of this as a linear transformation where input positions
        # are located on the x-axis and output positions on the y-axis.
        x1 = self.subtitles[p1[0]].start_seconds
        x2 = self.subtitles[p2[0]].start_seconds
        y1 = p1[1]
        y2 = p2[1]
        coefficient = (y2 - y1) / (x2 - x1)
        constant = - (coefficient * x1) + y1
        return coefficient, constant

    def _get_time_transform(self, p1, p2):
        """Return a formula for linear correction of positions.

        Return coefficient and constant, with which all subtitles should be
        scaled and shifted by to apply correction.
        """
        p1 = [p1[0], self.calc.time_to_seconds(p1[1])]
        p2 = [p2[0], self.calc.time_to_seconds(p2[1])]
        return self._get_seconds_transform(p1, p2)

    def adjust_durations_require(self, indices=None, *args, **kwargs):
        for index in indices or ():
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def adjust_durations(self,
                         indices=None,
                         speed=None,
                         lengthen=False,
                         shorten=False,
                         minimum=None,
                         maximum=None,
                         gap=None,
                         register=-1):
        """Lengthen or shorten durations by changing end positions.

        `indices` can be ``None`` to process all subtitles. `speed` is reading
        speed in seconds per character. `lengthen` is ``True`` to increase
        durations to match reading speed. `shorten` is ``True`` to decrease
        durations to match reading speed. `maximum` is the greatest allowed
        duration in seconds. `minimum` is the smallest allowed duration in
        seconds. `gap` is seconds to be left between consecutive subtitles.
        Using a gap of zero (or a bit more) is always a good idea if dealing
        with subtitles where overlapping is not desirable.

        Return changed indices.
        """
        new_indices = []
        new_subtitles = []
        for index in indices or self.get_all_indices():
            start = self.subtitles[index].start_seconds
            end = self.subtitles[index].end_seconds
            end_max = (self.subtitles[index + 1].start_seconds if
                       index < (len(self.subtitles) - 1) else 360000)

            if (speed is not None) and (lengthen or shorten):
                length = self.get_text_length(index, aeidon.documents.MAIN)
                optimal_duration = length / float(speed)
                if ((end - start) < optimal_duration) and lengthen:
                    end = start + optimal_duration
                if ((end - start) > optimal_duration) and shorten:
                    end = start + optimal_duration
            if (minimum is not None) and ((end - start) < minimum):
                end = start + minimum
            if (maximum is not None) and ((end - start) > maximum):
                end = start + maximum
            if (gap is not None) and ((end_max - end) < gap):
                end = max(start, end_max - gap)
            if end != self.subtitles[index].end_seconds:
                new_indices.append(index)
                subtitle = self.subtitles[index].copy()
                subtitle.end = end
                new_subtitles.append(subtitle)
        if new_indices:
            self.replace_positions(new_indices,
                                   new_subtitles,
                                   register=register)

            self.set_action_description(register, _("Adjusting durations"))
        return new_indices

    def convert_framerate_require(self, indices, *args, **kwargs):
        for index in indices or ():
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def convert_framerate(self,
                          indices,
                          framerate_in,
                          framerate_out,
                          register=-1):
        """Set the value of framerate and convert subtitles to it.

        `indices` can be ``None`` to process all subtitles.
        """
        self.set_framerate(framerate_in, register=None)
        new_subtitles = []
        indices = indices or self.get_all_indices()
        for index in indices:
            subtitle = self.subtitles[index].copy()
            subtitle.convert_framerate(framerate_out)
            new_subtitles.append(subtitle)
        self.set_framerate(framerate_out)
        self.replace_positions(indices, new_subtitles, register=register)
        self.group_actions(register, 2, _("Converting framerate"))

    @aeidon.deco.export
    @aeidon.deco.revertable
    def set_framerate(self, framerate, register=-1):
        """Set the value of framerate."""
        orig_framerate = self.framerate
        self.framerate = framerate
        self.calc = aeidon.Calculator(framerate)
        for subtitle in self.subtitles:
            subtitle.framerate = framerate
        action = self.new_revertable_action(register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Setting framerate")
        action.revert_function = self.set_framerate
        action.revert_args = (orig_framerate,)
        self.register_action(action)

    def shift_positions_require(self, indices, count, register=-1):
        for index in indices or ():
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def shift_positions(self, indices, value, register=-1):
        """Make subtitles appear earlier or later.

        `indices` can be ``None`` to process all subtitles. `value` should be a
        string for time, a float for seconds or an integer for frames.
        """
        new_subtitles = []
        indices = indices or self.get_all_indices()
        for index in indices:
            subtitle = self.subtitles[index].copy()
            subtitle.shift_positions(value)
            new_subtitles.append(subtitle)
        self.replace_positions(indices, new_subtitles, register=register)
        self.set_action_description(register, _("Shifting positions"))

    def transform_positions_require(self, indices, *args, **kwargs):
        for index in indices or ():
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def transform_positions(self, indices, p1, p2, register=-1):
        """Change positions by linear two-point corrections.

        `indices` can be ``None`` to process all subtitles. `p1` and `p2`
        should be tuples of index, position, where position should be a string
        for time, a float for seconds or an integer for frames.
        """
        if isinstance(p1[1], basestring):
            method = self._get_time_transform
        if isinstance(p1[1], int):
            method = self._get_frame_transform
        if isinstance(p1[1], float):
            method = self._get_seconds_transform
        coefficient, constant = method(p1, p2)
        new_subtitles = []
        indices = indices or self.get_all_indices()
        for index in indices:
            subtitle = self.subtitles[index].copy()
            subtitle.scale_positions(coefficient)
            subtitle.shift_positions(constant)
            new_subtitles.append(subtitle)
        self.replace_positions(indices, new_subtitles, register=register)
        self.set_action_description(register, _("Transforming positions"))



# Copyright (C) 2005-2008,200 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestModule(gaupol.TestCase):

    def test_char_to_px(self):
        assert gaupol.util.char_to_px(1) > 0
        assert gaupol.util.char_to_px(1, "monospace") > 0

    def test_delay_add(self):
        gaupol.util.delay_add(10, lambda: None)

    def test_document_to_text_field(self):
        doc2field = gaupol.util.document_to_text_field
        field = doc2field(aeidon.documents.MAIN)
        assert field == gaupol.fields.MAIN_TEXT
        field = doc2field(aeidon.documents.TRAN)
        assert field == gaupol.fields.TRAN_TEXT

    def test_get_font__set(self):
        gaupol.conf.editor.use_custom_font = True
        gaupol.conf.editor.custom_font = "Serif 12"
        assert gaupol.util.get_font() == "Serif 12"

    def test_get_font__unset(self):
        gaupol.conf.editor.use_custom_font = False
        assert gaupol.util.get_font() == ""
        gaupol.conf.editor.use_custom_font = True
        gaupol.conf.editor.custom_font = None
        assert gaupol.util.get_font() == ""

    def test_get_gst_version(self):
        reload(gaupol.util)
        assert gaupol.util.get_gst_version()

    def test_get_preview_command(self):
        gaupol.conf.preview.use_custom_command = True
        gaupol.conf.preview.force_utf_8 = False
        gaupol.util.get_preview_command()
        gaupol.conf.preview.use_custom_command = False
        gaupol.conf.preview.force_utf_8 = True
        gaupol.util.get_preview_command()
        gaupol.conf.preview.use_custom_command = False
        gaupol.conf.preview.force_utf_8 = False
        gaupol.util.get_preview_command()

    def test_get_pygst_version(self):
        reload(gaupol.util)
        assert gaupol.util.get_pygst_version()

    def test_get_text_view_size(self):
        text_view = gtk.TextView(gtk.TextBuffer())
        gaupol.util.get_text_view_size(text_view)

    def test_get_tree_view_size(self):
        tree_view = gtk.TreeView()
        scroller = gtk.ScrolledWindow()
        scroller.add(tree_view)
        gaupol.util.get_tree_view_size(tree_view)

    def test_gst_available(self):
        reload(gaupol.util)
        assert gaupol.util.gst_available()

    def test_gtkspell_available(self):
        reload(gaupol.util)
        assert gaupol.util.gtkspell_available()

    def test_lines_to_px(self):
        assert gaupol.util.lines_to_px(1) > 0

    def test_pocketsphinx_available(self):
        gaupol.util.pocketsphinx_available()

    def test_prepare_text_view__hide_lengths(self):
        gaupol.util.prepare_text_view(gtk.TextView())
        gaupol.conf.editor.show_lengths_edit = False
        gaupol.conf.editor.use_custom_font = True
        gaupol.conf.editor.custom_font = "Serif 12"

    def test_prepare_text_view__show_lengths(self):
        gaupol.util.prepare_text_view(gtk.TextView())
        gaupol.conf.editor.show_lengths_edit = True
        gaupol.conf.editor.use_custom_font = False
        gaupol.conf.editor.custom_font = ""

    def test_raise_default(self):
        self.raises(gaupol.Default, gaupol.util.raise_default, True)
        gaupol.util.raise_default(False)

    def test_scale_to_content__text_view(self):
        text_view = gtk.TextView()
        scroller = gtk.ScrolledWindow()
        scroller.add(text_view)
        gaupol.util.scale_to_content(text_view, 1, 2, 80, 10)

    def test_scale_to_content__tree_view(self):
        tree_view = gtk.TreeView()
        scroller = gtk.ScrolledWindow()
        scroller.add(tree_view)
        gaupol.util.scale_to_content(tree_view, 1, 2, 80, 10, "monospace")

    def test_scale_to_size(self):
        gaupol.util.scale_to_size(gtk.TextView(), 40, 5)
        gaupol.util.scale_to_size(gtk.TreeView(), 40, 5, "monospace")

    def test_separate_combo(self):
        combo_box = gtk.ComboBox()
        combo_box.set_row_separator_func(gaupol.util.separate_combo)

    def test_set_cursor_busy(self):
        window = gtk.Window()
        window.show_all()
        gaupol.util.set_cursor_normal(window)
        gaupol.util.set_cursor_busy(window)
        window.destroy()

    def test_set_cursor_normal(self):
        window = gtk.Window()
        window.show_all()
        gaupol.util.set_cursor_busy(window)
        gaupol.util.set_cursor_normal(window)
        window.destroy()

    def test_set_label_font(self):
        label = gtk.Label("testing...")
        gaupol.util.set_label_font(label, "Serif 12")

    def test_set_widget_font(self):
        label = gtk.Label("testing...")
        gaupol.util.set_label_font(label, "Serif 12")

    def test_show_uri(self):
        gaupol.util.show_uri(gaupol.HOMEPAGE_URL)

    def test_text_field_to_document(self):
        field2doc = gaupol.util.text_field_to_document
        doc = field2doc(gaupol.fields.MAIN_TEXT)
        assert doc == aeidon.documents.MAIN
        doc = field2doc(gaupol.fields.TRAN_TEXT)
        assert doc == aeidon.documents.TRAN



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialogs for applying linear tranfromations to positions."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._

__all__ = ("FrameTransformDialog", "TimeTransformDialog")


class PositionTransformDialog(gaupol.BuilderDialog):

    """Base class for dialogs for transforming positions."""

    _widgets = ("correction_hbox_1",
                "correction_hbox_2",
                "correction_label_1",
                "correction_label_2",
                "current_radio",
                "input_entry_1",
                "input_entry_2",
                "preview_button_1",
                "preview_button_2",
                "selected_radio",
                "subtitle_spin_1",
                "subtitle_spin_2",
                "text_label_1",
                "text_label_2")

    def __init__(self, parent, application):
        """Initialize a :class:`PositionTransformDialog` object."""
        gaupol.BuilderDialog.__init__(self, "transform-dialog.ui")
        self.application = application
        self._init_input_labels()
        self._init_sensitivities()
        self._init_sizes()
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_OK)

    def _get_target(self):
        """Return the selected target."""
        if self._selected_radio.get_active():
            return gaupol.targets.SELECTED
        if self._current_radio.get_active():
            return gaupol.targets.CURRENT
        raise ValueError("Invalid target radio state")

    def _init_input_labels(self):
        """Initialize non-editable input entries."""
        style = self._correction_label_1.get_style()
        text_color = style.fg[gtk.STATE_NORMAL]
        base_color = style.bg[gtk.STATE_NORMAL]
        for entry in (self._input_entry_1, self._input_entry_2):
            entry.modify_text(gtk.STATE_NORMAL, text_color)
            entry.modify_base(gtk.STATE_NORMAL, base_color)

    def _init_sensitivities(self):
        """Initialize sensitivities of widgets."""
        page = self.application.get_current_page()
        if page.project.video_path is None:
            self._preview_button_1.set_sensitive(False)
            self._preview_button_2.set_sensitive(False)
        if page.project.main_file is None:
            self._preview_button_1.set_sensitive(False)
            self._preview_button_2.set_sensitive(False)

    def _init_sizes(self):
        """Initialize widget sizes."""
        gaupol.util.scale_to_size(self._text_label_1, 40, -1)
        gaupol.util.scale_to_size(self._text_label_2, 40, -1)

    def _init_values(self):
        """Intialize default values for widgets."""
        page = self.application.get_current_page()
        self._subtitle_spin_1.set_value(1)
        self._subtitle_spin_2.set_value(len(page.project.subtitles))
        self._subtitle_spin_1.emit("value-changed")
        self._subtitle_spin_2.emit("value-changed")
        target = gaupol.conf.position_transform.target
        self._selected_radio.set_active(target == gaupol.targets.SELECTED)
        self._current_radio.set_active(target == gaupol.targets.CURRENT)
        rows = page.view.get_selected_rows()
        if (not rows) and (target == gaupol.targets.SELECTED):
            self._current_radio.set_active(True)
        self._selected_radio.set_sensitive(bool(rows))
        self._selected_radio.emit("toggled")

    def _init_widgets(self):
        """Initialize properties of widgets."""
        page = self.application.get_current_page()
        last_subtitle = len(page.project.subtitles)
        self._subtitle_spin_1.set_range(1, last_subtitle)
        self._subtitle_spin_2.set_range(1, last_subtitle)

    def _on_preview_button_1_clicked(self, *args):
        """Preview changes from the first point."""
        page = self.application.get_current_page()
        row = self._subtitle_spin_1.get_value_as_int() - 1
        doc = aeidon.documents.MAIN
        method = page.project.transform_positions
        target = self._get_target()
        rows = self.application.get_target_rows(target)
        point_1 = self._get_first_point()
        point_2 = self._get_second_point()
        args = (rows, point_1, point_2)
        self.application.preview_changes(page, row, doc, method, args)

    def _on_preview_button_2_clicked(self, *args):
        """Preview changes from the second point."""
        page = self.application.get_current_page()
        row = self._subtitle_spin_2.get_value_as_int() - 1
        doc = aeidon.documents.MAIN
        method = page.project.transform_positions
        target = self._get_target()
        rows = self.application.get_target_rows(target)
        point_1 = self._get_first_point()
        point_2 = self._get_second_point()
        args = (rows, point_1, point_2)
        self.application.preview_changes(page, row, doc, method, args)

    def _on_response(self, dialog, response):
        """Save target and transform positions."""
        gaupol.conf.position_transform.target = self._get_target()
        if response == gtk.RESPONSE_OK:
            self._transform_positions()

    def _on_selected_radio_toggled(self, *args):
        """Set subtitle values from selection."""
        page = self.application.get_current_page()
        rows = page.view.get_selected_rows()
        ranges = aeidon.util.get_ranges(rows)
        if (self._selected_radio.get_active()
            and len(rows) > 1
            and len(ranges) == 1
            and not self._output_edited()):
            self._subtitle_spin_1.set_value(rows[ 0] + 1)
            self._subtitle_spin_2.set_value(rows[-1] + 1)
        elif not self._output_edited():
            self._subtitle_spin_1.set_value(1)
            self._subtitle_spin_2.set_value(len(page.project.subtitles))

    def _transform_positions(self):
        """Transform positions in subtitles."""
        page = self.application.get_current_page()
        target = self._get_target()
        rows = self.application.get_target_rows(target)
        point_1 = self._get_first_point()
        point_2 = self._get_second_point()
        page.project.transform_positions(rows, point_1, point_2)


class FrameTransformDialog(PositionTransformDialog):

    """Dialog for applying linear tranfromations to frames."""

    __metaclass__ = aeidon.Contractual

    def __init__(self, parent, application):
        """Initialize a :class:`FrameTransformDialog` object."""
        PositionTransformDialog.__init__(self, parent, application)
        self._output_spin_1 = gtk.SpinButton()
        self._output_spin_2 = gtk.SpinButton()
        self._init_widgets()
        self._init_values()

    def _get_first_point_ensure(self, value):
        assert isinstance(value[1], int)

    def _get_first_point(self):
        """Return row, output frame of the first sync point."""
        return (self._subtitle_spin_1.get_value_as_int() - 1,
                self._output_spin_1.get_value_as_int())

    def _get_second_point_ensure(self, value):
        assert isinstance(value[1], int)

    def _get_second_point(self):
        """Return row, output frame of the second sync point."""
        return (self._subtitle_spin_2.get_value_as_int() - 1,
                self._output_spin_2.get_value_as_int())

    def _init_widgets(self):
        """Initialize properties of widgets."""
        PositionTransformDialog._init_widgets(self)
        self._input_entry_1.set_width_chars(6)
        self._input_entry_2.set_width_chars(6)
        self._output_spin_1.set_digits(0)
        self._output_spin_2.set_digits(0)
        self._output_spin_1.set_increments(1, 10)
        self._output_spin_2.set_increments(1, 10)
        self._output_spin_1.set_range(0, 999999)
        self._output_spin_2.set_range(0, 999999)
        self._correction_hbox_1.pack_start(self._output_spin_1)
        self._correction_hbox_2.pack_start(self._output_spin_2)
        self._correction_hbox_1.show_all()
        self._correction_hbox_2.show_all()
        self._correction_label_1.set_mnemonic_widget(self._output_spin_1)
        self._correction_label_2.set_mnemonic_widget(self._output_spin_2)

    def _on_subtitle_spin_1_value_changed(self, spin_button):
        """Update subtitle data in widgets."""
        page = self.application.get_current_page()
        row = spin_button.get_value_as_int() - 1
        subtitle = page.project.subtitles[row]
        self._input_entry_1.set_text(str(subtitle.start_frame))
        self._output_spin_1.set_value(subtitle.start_frame)
        text = subtitle.main_text.replace("\n", " ")
        text = aeidon.re_any_tag.sub("", text)
        self._text_label_1.set_text(text)
        self._text_label_1.set_tooltip_text(subtitle.main_text)
        self._subtitle_spin_2.props.adjustment.props.lower = row + 2

    def _on_subtitle_spin_2_value_changed(self, spin_button):
        """Update subtitle data in widgets."""
        page = self.application.get_current_page()
        row = spin_button.get_value_as_int() - 1
        subtitle = page.project.subtitles[row]
        self._input_entry_2.set_text(str(subtitle.start_frame))
        self._output_spin_2.set_value(subtitle.start_frame)
        text = subtitle.main_text.replace("\n", " ")
        text = aeidon.re_any_tag.sub("", text)
        self._text_label_2.set_text(text)
        self._text_label_2.set_tooltip_text(subtitle.main_text)
        self._subtitle_spin_1.props.adjustment.props.upper = row

    def _output_edited(self):
        """Return ``True`` if output of either point has been edited."""
        in_1 = int(self._input_entry_1.get_text())
        in_2 = int(self._input_entry_2.get_text())
        out_1 = self._output_spin_1.get_value_as_int()
        out_2 = self._output_spin_2.get_value_as_int()
        return (out_1 != in_1 or out_2 != in_2)

class TimeTransformDialog(PositionTransformDialog):

    """Dialog for applying linear tranfromations to times."""

    __metaclass__ = aeidon.Contractual

    def __init__(self, parent, application):
        """Initialize a :class:`TimeTransformDialog` object."""
        PositionTransformDialog.__init__(self, parent, application)
        self._output_entry_1 = gaupol.TimeEntry()
        self._output_entry_2 = gaupol.TimeEntry()
        self._init_widgets()
        self._init_values()

    def _get_first_point_ensure(self, value):
        assert isinstance(value[1], basestring)

    def _get_first_point(self):
        """Return row, output time of the first sync point."""
        return (self._subtitle_spin_1.get_value_as_int() - 1,
                self._output_entry_1.get_text())

    def _get_second_point_ensure(self, value):
        assert isinstance(value[1], basestring)

    def _get_second_point(self):
        """Return row, output time of the second sync point."""
        return (self._subtitle_spin_2.get_value_as_int() - 1,
                self._output_entry_2.get_text())

    def _init_widgets(self):
        """Initialize properties of widgets."""
        PositionTransformDialog._init_widgets(self)
        self._input_entry_1.set_width_chars(13)
        self._input_entry_2.set_width_chars(13)
        self._correction_hbox_1.pack_start(self._output_entry_1)
        self._correction_hbox_2.pack_start(self._output_entry_2)
        self._correction_hbox_1.show_all()
        self._correction_hbox_2.show_all()
        self._correction_label_1.set_mnemonic_widget(self._output_entry_1)
        self._correction_label_2.set_mnemonic_widget(self._output_entry_2)

    def _on_subtitle_spin_1_value_changed(self, spin_button):
        """Update subtitle data in widgets."""
        page = self.application.get_current_page()
        row = spin_button.get_value_as_int() - 1
        subtitle = page.project.subtitles[row]
        self._input_entry_1.set_text(subtitle.start_time)
        self._output_entry_1.set_text(subtitle.start_time)
        text = subtitle.main_text.replace("\n", " ")
        text = aeidon.re_any_tag.sub("", text)
        self._text_label_1.set_text(text)
        self._text_label_1.set_tooltip_text(subtitle.main_text)
        self._subtitle_spin_2.props.adjustment.props.lower = row + 2
        # Allow the glib.idle_add-using TimeEntry to update.
        gaupol.util.iterate_main()

    def _on_subtitle_spin_2_value_changed(self, spin_button):
        """Update subtitle data in widgets."""
        page = self.application.get_current_page()
        row = spin_button.get_value_as_int() - 1
        subtitle = page.project.subtitles[row]
        self._input_entry_2.set_text(subtitle.start_time)
        self._output_entry_2.set_text(subtitle.start_time)
        text = subtitle.main_text.replace("\n", " ")
        text = aeidon.re_any_tag.sub("", text)
        self._text_label_2.set_text(text)
        self._text_label_2.set_tooltip_text(subtitle.main_text)
        self._subtitle_spin_1.props.adjustment.props.upper = row
        # Allow the glib.idle_add-using TimeEntry to update.
        gaupol.util.iterate_main()

    def _output_edited(self):
        """Return ``True`` if output of either point has been edited."""
        in_1 = self._input_entry_1.get_text()
        in_2 = self._input_entry_2.get_text()
        out_1 = self._output_entry_1.get_text()
        out_2 = self._output_entry_2.get_text()
        return (out_1 != in_1 or out_2 != in_2)



# Copyright (C) 2005-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Storing text to the clipboard and pasting from it."""

from __future__ import division

import aeidon
_ = aeidon.i18n._


class ClipboardAgent(aeidon.Delegate):

    """Storing text to the clipboard and pasting from it."""

    __metaclass__ = aeidon.Contractual

    def copy_texts_require(self, indices, doc):
        for index in indices:
            assert 0 <= index < len(self.subtitles)

    def copy_texts_ensure(self, value, indices, doc):
        texts = self.clipboard.get_texts()
        assert len(texts) == len(range(indices[0], indices[-1] + 1))
        while None in texts:
            texts.remove(None)
        assert len(texts) == len(indices)

    @aeidon.deco.export
    def copy_texts(self, indices, doc):
        """Copy texts to the clipboard."""
        self.clipboard.clear()
        for index in range(min(indices), max(indices) + 1):
            subtitle = self.subtitles[index]
            text = (subtitle.get_text(doc) if index in indices else None)
            self.clipboard.append(text)

    def cut_texts_require(self, indices, doc, register=-1):
        for index in indices:
            assert 0 <= index < len(self.subtitles)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def cut_texts(self, indices, doc, register=-1):
        """Cut texts to the clipboard."""
        self.copy_texts(indices, doc)
        self.clear_texts(indices, doc, register=register)
        self.set_action_description(register, _("Cutting texts"))

    def paste_texts_require(self, index, doc, register=-1):
        assert 0 <= index <= len(self.subtitles)
        assert self.clipboard.get_texts()

    @aeidon.deco.export
    @aeidon.deco.revertable
    def paste_texts(self, index, doc, register=-1):
        """Paste texts from the clipboard and return pasted indices."""
        texts = self.clipboard.get_texts()
        length = len(self.subtitles)
        new_count = len(texts) - (length - index)
        if new_count > 0:
            self.insert_subtitles(range(length, length + new_count),
                                  register=register)

        indices = [index + i for i in range(len(texts))
                   if texts[i] is not None]

        new_texts = [x for x in texts if x is not None]
        self.replace_texts(indices, doc, new_texts, register=register)
        if new_count > 0:
            self.group_actions(register, 2, "")
        self.set_action_description(register, _("Pasting texts"))
        return tuple(indices)
